# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-08 16:28:17
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-21 14:16:49

from typing import List, Union, Dict
from dataclasses import dataclass
from abc import ABC
from .general import (
    delete,
    get_extension,
    get_name,
    make_dir,
    run_cmd,
    get_cmd_status,
    is_directory,
    paths_in_dir,
    CMD_STATUS,
)
from gailbot.shared.utils.logger import makelogger
from pydub import AudioSegment
from pydub.silence import detect_leading_silence
from moviepy.editor import *

MERGED_FILE_NAME = "merged"
logger = makelogger("media")


@dataclass
class Stream(ABC):
    """Abstract class that defines the representation of a media stream

    Attributes
    name : str
        the name of the stream file
    source : Union [List[str], List[Stream]]
        the source of the stream, which can either be a list of paths to a media
        file or a list of media Stream
    extension : str
        the file extension, which tells the file type
    """

    source: Union[List[str], List["Stream"]]
    name: str
    extension: str


@dataclass
class AudioStream(Stream):
    """A subclass of Steam class which defines the representation of
        audio stream

    Attributes
    segment : AudioSegment
        stores source of an audio segment

    """

    segment: AudioSegment

    def __repr__(self):
        return f"audio stream {self.name}\n"


@dataclass
class VideoStream(Stream):
    """A subclass of Steam class which defines the representation of
    Video stream

    """

    clip: VideoFileClip

    def __repr__(self):
        return f"video stream {self.name}\n"


class AudioHandler:
    """Implement a class that contains functions to handle audio file

    Attributes
        _SUPPORTED_FORMATS: List[str]
            a list of supported audio format
        _DEFAULT_FORMAT:
            the default audio format
    """

    _SUPPORTED_FORMATS = ["mp3", "mpeg", "opus", "wav", "aac"]
    _DEFAULT_FORMAT = "wav"

    def __repr__(self) -> str:
        return "Audio Stream handler"

    @property
    def supported_formats(self) -> List[str]:
        """return a list of string with supported audio format"""
        return self._SUPPORTED_FORMATS

    @staticmethod
    def is_supported(path: str) -> bool:
        """tell if a audio file is supported by the AudioHandler

        Args:
            path (str): a path to a file

        Returns:
            bool: return true of the file is supported
        """
        return get_extension(path) in AudioHandler._SUPPORTED_FORMATS

    def read_file(self, path: str) -> AudioStream:
        """read the audio file data

        Args:
            path (str): a path to a file

        Returns:
            AudioStream: return the audio file data as an AudioStream object
                         if the file type is supported, return nothing
                         if the file type is unsupported
        """
        format = get_extension(path)
        if format not in self.supported_formats:
            raise Exception(f"Format {format} not supported")

        segment = AudioSegment.from_file(path, format=format)
        return AudioStream(
            source=[path], name=get_name(path), extension=format, segment=segment
        )

    def record(
        self,
        name: str,
        out_dir: str,
        duration_sec: float = -1,
    ) -> AudioStream:
        """record the audio file"""
        raise NotImplementedError()

    def write_stream(
        self, stream: AudioStream, outdir: str, name: str = None, format: str = None
    ) -> str:
        """write a stream with the given format. If not format or name is
            specified, uses the stream's default name and format.

        Args:
            stream: AudioStream
                stores the audio file data as an instance of AudioStream
            outdir: str
                a path that stores the output directory
            name: str (Optional)
                file name, default to the name of stream
            format: str (Optional)
                file format, default to the extension of  stream

        Return:
            the file path
        """
        if format not in self.supported_formats:
            raise Exception(f"Format {format} not supported")
        # Construct output file path
        name = stream.name if name == None else name
        ext = stream.extension if format == None else format
        make_dir(outdir, overwrite=False)
        path = f"{outdir}/{name}.{ext}"
        stream.segment.export(path, format=ext)
        return path

    def info(self, stream: AudioStream) -> Dict:
        """Get information about the given audio stream

        Args:
            stream: AudioStream
                stores the audio file data as an instance of AudioStream

        Return:
            a dictionary with the stream information
        """
        segment = stream.segment
        return {
            "name": stream.name,
            "format": stream.extension,
            "source": stream.source,
            "decibels_relative_to_full_scale": segment.dBFS,
            "channels": segment.channels,
            "sample_width": segment.sample_width,
            "frame_rate": segment.frame_rate,
            "frame_width": segment.frame_width,
            "root_mean_square": segment.rms,
            "highest_amplitude": segment.max,
            "duration_seconds": segment.duration_seconds,
            "num_frames": segment.frame_count(),
        }

    def change_volume(self, stream: AudioStream, change_db: float) -> AudioStream:
        """
        Apply the specified gain to the audio stream.
        Args:
            stream: AudioStream
                stores the audio file data as an instance of AudioStream
            change_db: float
                the targeting value of the volume

        Return:
            the resulting audio stream as an instance of AudioStream
        """
        segment = stream.segment.apply_gain(change_db)
        stream.segment = segment
        return stream

    def mono_to_stereo(
        self, left_stream: AudioStream, right_stream: AudioStream
    ) -> AudioStream:
        """
        Get a new stereo stream from the two mono streams
        Args:
            left_stream: AudioStream
                stores the data of a mono audio file
            right_stream: AudioStream
                stores the data of a mono audio file

        Return:
            the resulting audio stream as an instance of AudioStream
        """
        stereo_name = f"{left_stream.name}_{right_stream.name}_stereo"
        sources = [left_stream, right_stream]
        segment = AudioSegment.from_mono_audiosegments(
            left_stream.segment, right_stream.segment
        )
        return AudioStream(sources, stereo_name, self._DEFAULT_FORMAT, segment)

    def stereo_to_mono(self, stream: AudioStream) -> List[AudioStream]:
        """
        Convert the given stereo stream to mono.

        Args:
            stream: AudioStream
                stores the data of a stereo audio file

        Returns:
            a list of two mono audio file

        """
        left_segment, right_segment = stream.segment.split_to_mono()
        left_stream = AudioStream(
            [stream], f"{stream.name}_left", self._DEFAULT_FORMAT, left_segment
        )
        right_stream = AudioStream(
            [stream], f"{stream.name}_right", self._DEFAULT_FORMAT, right_segment
        )
        return left_stream, right_stream

    def concat(self, streams: List[AudioStream]) -> AudioStream:
        """
        Concat the given stream end to end, start to finish, into a single stream.

        Args:
            streams: List[AudioStream]
                A list of audio streams

        Returns:
            the resulting audio stream that is composed by concatenating the
            input streams
        """
        concatenated = AudioSegment.empty()
        for stream in streams:
            concatenated += stream.segment
        name = "_".join([stream.name for stream in streams])
        name += "_concatenated"
        return AudioStream(
            source=[streams],
            name=name,
            extension=self._DEFAULT_FORMAT,
            segment=concatenated,
        )

    def overlay(
        self,
        left_stream: AudioStream,
        right_stream: AudioStream,
        loop_shorter_stream: bool = False,
    ) -> AudioStream:
        """
        Overlay two audio streams on top of each other

        Args:
            left_stream:AudioStream
                stores one audio file data that will be overlayed
            right_stream:AudioStream
                stores one audio file data that will be overlayed
            loop_shorter_stream: bool
                if true, the shorter audio file will be looped until it reaches
                the length of the longer audio file
        """
        # Determine which segment is longer
        if left_stream.segment.duration_seconds > right_stream.segment.duration_seconds:
            segments = [left_stream.segment, right_stream.segment]
        else:
            segments = [right_stream.segment, left_stream.segment]
        if loop_shorter_stream:
            overlaid_segment = segments[0].overlay(segments[1], loop=True)
        else:
            # Create a silent stream to cover duration difference
            duration_diff = segments[0].duration_seconds - segments[1].duration_seconds
            silence = AudioSegment.silent(duration=duration_diff)
            segments[1] += silence
            overlaid_segment = segments[0].overlay(segments[1])
        name = f"{left_stream.name}_{right_stream.name}_overlaid"
        return AudioStream(
            [left_stream, right_stream], name, self._DEFAULT_FORMAT, overlaid_segment
        )

    def reverse(self, stream: AudioStream) -> AudioStream:
        """
        Reverse the given audio stream

        Args:
            stream: AudioStream
                stores the data of an audio stream
        Returns:
            the resulting audio stream after reversed
        """
        reversed_segment = stream.segment.reverse()
        stream.segment = reversed_segment
        return stream

    def chunk(self, stream: AudioStream, chunk_duration_s: float) -> List[AudioStream]:
        """
        Generate chunks of the given audio with the provided duration.

        Args:
        stream: AudioStream
                stores the data of an audio stream

        Returns:
            a list of audio stream, which are the chunks of the given audio
            stream

        """
        assert chunk_duration_s > 0, f"Duration must be positive"
        # Simply return original stream if no chunking possible.
        if chunk_duration_s > stream.segment.duration_seconds:
            return [stream]

        duration_ms = chunk_duration_s * 1000
        chunks = list()
        for i, chunk_segment in enumerate(stream.segment[:: int(duration_ms)]):
            name = f"{stream.name}_{chunk_duration_s}_chunk_{i}"
            chunk = AudioStream([stream], name, self._DEFAULT_FORMAT, chunk_segment)
            chunks.append(chunk)
        return chunks

    @staticmethod
    def convert_to_16bit_wav(input_path, output_path):
        if os.path.isfile(output_path):
            delete(output_path)
        try:
            result = run_cmd(
                [
                    "ffmpeg",
                    "-i",
                    f'"{input_path}"',
                    "-acodec",
                    "pcm_s16le",
                    "-ar",
                    "16000",
                    f'"{output_path}"',
                ]
            )
            assert result == 0
            return output_path
        except Exception as e:
            logger.warn(
                f"failed to convert to 16 bit wav fail use original file instead"
            )
            return input_path

    @staticmethod
    def compress_to_opus(audio_path: str, output_dir: str):
        """compress the audio file stored in audio_path to output_dir,
            with the same audio file name with opus format

        Args:
            audio_path (str): the path to original file
            output_dir (str): the path to the directory where the compressed file
                              will be stored

        Raises:
            ChildProcessError:
            ChildProcessError:
            ProcessLookupError:

        Returns:
            str: the path to the compressed file
        """
        try:
            out_path = "{}/{}.opus".format(output_dir, get_name(audio_path))
            if os.path.isfile(out_path):
                delete(out_path)
            result = run_cmd(
                [
                    "ffmpeg",
                    "-y",
                    "-i",
                    f'"{audio_path}"',
                    "-strict",
                    "-2",
                    f'"{out_path}"',
                ]
            )
            assert result == 0
            return out_path
        except Exception as e:
            logger.warn(
                f" fail to compress to opus file due to {e}, use original file instead"
            )
            return audio_path

    @staticmethod
    def chunk_audio_to_outpath(
        audio_path: str, output_path: str, chunk_duration: int
    ) -> List[str]:
        """given the audio path, chunking the audio into a series of audio
            segment

        Args:
            audio_path[str]: the file path to the audio file
            output_path[str]: the output of the chunked file
            duration[int]: the length of each chunk

        Return:
            Union[List[str], bool]: return a list of the audio path to the
                                    chunked audios in order
        """
        if not is_directory(output_path):
            make_dir(audio_path)
        basename = get_name(audio_path)
        dir = os.path.join(output_path, f"{basename}_audio_chunks")
        make_dir(dir)
        idx = 0
        try:
            mediaHandler = MediaHandler()
            stream = mediaHandler.read_file(audio_path)
            chunks = mediaHandler.chunk(stream, chunk_duration)
            for chunk in chunks:
                mediaHandler.write_stream(
                    chunk,
                    dir,
                    name=f"{basename}-{idx}",
                    format=get_extension(audio_path),
                )
                idx += 1

            audio_chunks = paths_in_dir(dir)
            audio_chunks = sorted(audio_chunks, key=lambda file: (len(file), file))

        except Exception as e:
            logger.error(e, exc_info=e)
        else:

            return audio_chunks

    def overlay_audios(
        self, audios: List[str], output_path: str, name=MERGED_FILE_NAME
    ):
        res = self.read_file(audios[0])
        name = MERGED_FILE_NAME
        if len(audios) == 1:
            res = self.read_file(audios[0])
        else:
            for i in range(1, len(audios)):
                if AudioHandler.is_supported(audios[i]):
                    nxt = self.read_file(audios[i])
                    res = self.overlay(res, nxt)
        return self.write_stream(
            res, output_path, name=name, format=get_extension(audios[0])
        )


class VideoHandler:
    _SUPPORTED_FORMATS = ["mxf", "mp4"]
    _BASE_FORMAT = "mp4"

    @property
    def supported_formats(self) -> List[str]:
        return self._SUPPORTED_FORMATS

    @staticmethod
    def is_supported(path: str) -> bool:
        return get_extension(path) in ["mp4"]

    def read_file(self, path: str) -> VideoStream:
        """read the video file data

        Args:
            path (str): a path to a file

        Returns:
            VideoStream: return the video file data as a VideoStream object
                         if the file type is unsupported, return nothing
        """
        if not VideoHandler.is_supported(path):
            return
        format = get_extension(path)
        clip = VideoFileClip(path)
        return VideoStream(
            source=[path], name=get_name(path), extension=format, clip=clip
        )

    def record(
        self,
        name: str,
        out_dir: str,
        duration_sec: float = -1,
    ) -> VideoStream:
        raise NotImplementedError()

    def write_stream(
        self, stream: VideoStream, out_dir: str, name: str = None, format: str = None
    ) -> str:
        if format not in self.supported_formats:
            raise Exception(f"Format {format} not supported")
        # Construct output file path
        name = stream.name if name == None else name
        extension = stream.extension if format == None else format
        make_dir(out_dir, overwrite=False)
        path = f"{out_dir}/{name}.{extension}"
        stream.clip.write_videofile(path)
        return path

    def info(self, stream: VideoStream) -> Dict:
        clip = stream.clip
        return {
            "name": stream.name,
            "format": stream.extension,
            "source": stream.source,
            "size": clip.size,
            "frames_per_second": clip.fps,
            "duration_seconds": clip.duration,
        }

    def change_volume(self, stream: VideoStream, change_db: float) -> VideoStream:
        raise NotImplementedError()

    def extract_audio(self, stream: VideoStream) -> AudioStream:
        audioClip = stream.clip.audio
        path = f"{stream.name}.mp3"
        audioClip.write_audiofile(path)
        audioHandler = AudioHandler()
        audioStream = audioHandler.read_file(path)
        os.remove(path)
        return audioStream

    def remove_audio(self, stream: VideoStream) -> VideoStream:
        stream.clip = stream.clip.withoutaudio()
        return stream


class MediaHandler:
    def __init__(self):
        """
        Initializes an object of the MediaHandler class, an abstraction of the AudioHandler and
        VideoHandler classes.
        """
        self.audio_h = AudioHandler()
        self.video_h = VideoHandler()
        self._SUPPORTED_FORMATS = (
            self.audio_h.supported_formats + self.video_h.supported_formats
        )

    def __repr__(self) -> str:
        return "Media Stream Handler"

    @property
    def supported_formats(self) -> List[str]:
        """
        Accesses list of supported formats.

        Args:
            Self
        Returns:
            A list of strings representing supported formats of the given MediaHandler object
        """
        return self._SUPPORTED_FORMATS

    #### Audio and Video
    @staticmethod
    def is_supported(path: str) -> bool:
        return AudioHandler.is_supported(path) or VideoHandler.is_supported(path)

    @staticmethod
    def is_audio(path: str) -> bool:
        """
        Determines if the given file is an audio file.

        Args:
            path: Path to given file in string form

        Returns:
            True if the given file is an audio file, false if the given file
            is not an audio file.
        """
        return AudioHandler.is_supported(path)

    @staticmethod
    def is_video(path: str) -> bool:
        """
        Determines if the given file is a video file.

        Args:
            path: Path to given file in string form

        Returns:
            True if the given file is a video file, false if the given file
            is not a video file.
        """
        return VideoHandler.is_supported(path)

    def read_file(self, path: str) -> Stream:
        """
        Reads inputted file.

        Args:
            path: Path to given file in string form

        Returns:
            An AudioStream or VideoStream containing the read stream.
            Raises exception if path is not supported.

        """
        if get_extension(path) in self.audio_h.supported_formats:
            return self.audio_h.read_file(path)
        elif get_extension(path) in self.video_h.supported_formats:
            return self.video_h.read_file(path)
        else:
            raise Exception(f"ERROR: Format not supported for file: {path}")

    def record(
        self,
        name: str,
        out_dir: str,
        out_stream_type: Union[AudioStream, VideoStream],
        duration_sec: float = -1,
    ) -> Stream:
        """
        Records audio or video file from inputted stream.

        Args:
            name: File name as a string.
            out_dir: Path of the desired output directory as a string.
            out_stream_type: AudioStream or VideoStream, depending on the inputted file.
            duration_sec: Float containing the duration to record.

        Returns:
            Currently will raise NotImplementedError.
        """
        if out_stream_type == AudioStream:
            return self.audio_h.record(name, out_dir, duration_sec)
        else:
            return self.video_h.record(name, out_dir, duration_sec)

    def write_stream(
        self, stream: Stream, out_dir: str, name: str = None, format: str = None
    ) -> str:
        """
        Writes audio or video stream to given output directory.

        Args:
            stream: Inputted stream, of type AudioStream or VideoStream.
            out_dir: Path of the desired output directory as a string.
            name: File name as a string.
            format: Audio format as a string.

        Returns:
            Output as type stream.
        """
        return self._get_handler(stream).write_stream(stream, out_dir, name, format)

    def info(self, stream: Stream) -> Dict:
        """
        Accesss audio or video information

        Args:
            stream: Inputted stream, of type AudioStream or VideoStream.

        Returns:
            Information pertaining to the inputted stream in the form of a dictionary
            with different data fields.
        """
        return self._get_handler(stream).info(stream)

    def change_volume(self, stream: Stream, change_db: float) -> Stream:
        """
        Change audio or video information by given scale

        Args:
            stream: Inputted stream, of type AudioStream or VideoStream.
            change_db: Float value representing the decibals by which to change the volume.

        Returns:
            The resulting stream as an instance of AudioStream or VideoStream.
        """
        return self._get_handler(stream).change_volume(stream, change_db)

    ### Audio Methods
    def mono_to_stereo(
        self, left_stream: AudioStream, right_stream: AudioStream
    ) -> AudioStream:
        """
        Converts the given mono stream to stereo.

         Args:
            left_stream: AudioStream
                stores the data of a mono audio file
            right_stream: AudioStream
                stores the data of a mono audio file

        Returns:
            the resulting stereo audio stream as an instance of AudioStream.

        """
        return self.audio_h.mono_to_stereo(left_stream, right_stream)

    def stereo_to_mono(self, stream: AudioStream) -> List[AudioStream]:
        """
        Converts the given stereo stream to mono.

        Args:
            stream: AudioStream
                stores the data of a stereo audio file

        Returns:
            a list of two mono audio streams representing the two files.
        """
        return self.audio_h.stereo_to_mono(stream)

    def concat(self, streams: List[AudioStream]) -> AudioStream:
        """
        Concatenates the given stream end to end, start to finish, into a single stream.

        Args:
            streams: List of AudioStreams of which to concatenate together

        Returns:
            AudioStream representing the concatenated audio.
        """
        return self.audio_h.concat(streams)

    def overlay(
        self,
        left_stream: AudioStream,
        right_stream: AudioStream,
        loop_shorter_stream: bool = False,
    ) -> AudioStream:
        """
        Overlays two audio streams on top of each other

        Args:
            left_stream:AudioStream
                stores one audio file data that will be overlayed
            right_stream:AudioStream
                stores one audio file data that will be overlayed
            loop_shorter_stream: bool
                if true, the shorter audio file will be looped until it reaches
                the length of the longer audio file
        Returns:
            AudioStream representing the overlaid audio files.
        """
        return self.audio_h.overlay(left_stream, right_stream, loop_shorter_stream)

    def reverse(self, stream: AudioStream) -> AudioStream:
        """
        Reverse the given audio stream in place

        Args:
            stream: AudioStream to reverse

        Returns:
            AudioStream representing the reversed audio.
        """
        return self.audio_h.reverse(stream)

    def chunk(self, stream: AudioStream, chunk_duration_s: float) -> List[AudioStream]:
        """
        Generates chunks of the given audio with the provided duration.

        Args:
            stream: AudioStream for which to generate chunks
            chunk_duration_s: float representing the duration to make each chunk

        Returns:
            A list of AudioStreams containing each generated chunk
        """
        return self.audio_h.chunk(stream, chunk_duration_s)

    ### Video Methods
    def extract_audio(self, stream: VideoStream) -> AudioStream:
        raise NotImplementedError

    def remove_audio(self, stream: VideoStream) -> VideoStream:
        raise NotImplementedError

    def _get_handler(self, stream: Stream) -> Union[AudioHandler, VideoHandler]:
        """
        Returns the handler of a given stream; either Audio or Video

        Args:
            stream for which to generate the handler

        Returns:
            AudioHandler or VideoHandler, depending on the type of the given stream."""
        return self.audio_h if isinstance(stream, AudioStream) else self.video_h

    @staticmethod
    def convert_to_16bit_wav(input_path, output_path):
        return AudioHandler.convert_to_16bit_wav(input_path, output_path)

    @staticmethod
    def compress_to_opus(media_path: str, output_dir: str):
        """compress the  file stored in media_path to output_dir,
            with the same  file name with compressed format

        Args:
            media_path (str): the path to original file
            output_dir (str): the path to the directory where the compressed file
                              will be stored

        Raises:
            ChildProcessError:
            ChildProcessError:
            ProcessLookupError:

        Returns:
            str: the path to the compressed file
        """
        return AudioHandler.compress_to_opus(media_path, output_dir)

    @staticmethod
    def chunk_audio_to_outpath(
        input_path: str, output_path: str, duration: int
    ) -> List[str]:
        """given the audio path, chunking the audio into a series of audio
            segments

        Args:
            audiopath[str]: the file path to the audio file
            output_path[str]: the output of the chunked file
            duration[int]: the length of each chunk

        Return:
            Union[List[str], bool]: return a list of the audio path to the
                                    chunked audios in order
        """
        return AudioHandler.chunk_audio_to_outpath(input_path, output_path, duration)

    @staticmethod
    def remove_prelude_silence(input_path: str, outdir: str) -> str:
        ouput_path = os.path.join(
            outdir, f"{get_name(input_path)}.{get_extension(input_path)}"
        )
        sound = AudioSegment.from_file(input_path)
        leading_silence = detect_leading_silence(sound)
        trim_ed: AudioSegment = sound[leading_silence:]
        trim_ed.export(ouput_path, "wav")
        return ouput_path



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-01-29 13:26:17
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-21 14:31:59
import os.path
from dataclasses import dataclass, asdict
from typing import List, Dict, Any, Optional

from gailbot.engineManager.engine.engineProvider import EngineProvider
from gailbot.pluginSuiteManager.pluginSuiteManager import PluginSuiteManager
from gailbot.shared.exception.serviceException import (
    ProfileNotFound,
    DuplicateProfile,
    RemoveDefaultProfile,
    EngineNotFound,
)
from gailbot.shared.utils.general import (
    is_file,
    delete,
    write_toml,
    paths_in_dir,
    get_name,
    read_toml,
)
from gailbot.shared.utils.logger import makelogger
from gailbot.engineManager.engineManager import EngineManager
from gailbot.pluginSuiteManager.suite.suite import PluginSuite


@dataclass(init=True)
class ProfileSetting:
    """
    stores the raw data for a profile setting
    """

    engine_setting_name: str
    plugin_suite_setting: Dict[str, Optional[Dict[str, List[str]] | List[str]]]


@dataclass(init=True)
class ProfileObject:
    """
    Attributes:
        name: name of the profile
        profile_data: the raw setting data
        engine_provider: an instance of EngineProvider used by current profile
        plugin_suites: a dictionary maps the plugin_suites and the selected plugins
                      (stored as a dependency graph) used by current profile
    """

    name: str
    profile_data: ProfileSetting
    engine_provider: EngineProvider
    plugin_suites: Dict[PluginSuite, Dict[str, List[str]]]


class ProfileManager:
    logger = makelogger("profileManager")
    _default_name = "Default"
    _default_data = ProfileSetting(
        engine_setting_name=EngineManager.default_name,
        plugin_suite_setting={},
    )

    def __init__(
        self,
        workspace: str,
        engine_manager: EngineManager,
        plugin_suite_manager: PluginSuiteManager,
    ):
        self.workspace = workspace
        self.engineManager = engine_manager
        self.pluginSuiteManager = plugin_suite_manager
        self.profiles: Dict[str, ProfileSetting] = dict()
        try:
            self._load_existing_profile()
            self._load_default_profile()
        except Exception as e:
            self.logger.error(e, exc_info=True)

    def add_new_profile(self, name: str, data: Dict[str, Any] | ProfileSetting):
        if self.is_profile(name):
            raise DuplicateProfile(name)

        if isinstance(data, ProfileSetting):
            profile_setting = data
            raw_data = asdict(data)
        else:
            profile_setting = ProfileSetting(**data)
            raw_data = data

        if not self.engineManager.is_engine(profile_setting.engine_setting_name):
            raise EngineNotFound(profile_setting.engine_setting_name)

        self.profiles[name] = profile_setting

        # save profile to disk
        path = os.path.join(self.workspace, f"{name}.toml")
        if is_file(path):
            delete(path)

        write_toml(path=path, data=raw_data)

    @property
    def default_profile_name(self):
        return self._default_name

    @property
    def available_profiles(self) -> List[str]:
        """
        Returns
        -------
        a list of profile names
        """
        return list(self.profiles.keys())

    def is_profile(self, name):
        return name in self.profiles

    def remove_profile(self, name):
        """
        Raises
        -------
        ProfileNotFound

        Returns
        -------
        Remove profile identified by name
        """
        if not self.is_profile(name):
            raise ProfileNotFound(name)
        if name == self.default_profile_name:
            raise RemoveDefaultProfile()

        path = os.path.join(self.workspace, f"{name}.toml")
        if is_file(path):
            delete(path)
        del self.profiles[name]

    def update_profile(self, name, data: Dict[str, Any] | ProfileSetting):
        """
        Returns
        -------
        Update the current profile identified by name with new data
        """
        if not self.is_profile(name):
            raise ProfileNotFound(name)
        path = os.path.join(self.workspace, f"{name}.toml")
        if is_file(path):
            delete(path)
        del self.profiles[name]
        self.add_new_profile(name, data)

    def get_profile_object(self, profile_name: str) -> ProfileObject:
        """
        Return the profile object that contains initialized engine and plugin suite instances
        """
        profile_setting: ProfileSetting = self.profiles[profile_name]
        engine_provider = self.engineManager.get_engine_provider(
            profile_setting.engine_setting_name
        )
        suites = {
            self.pluginSuiteManager.get_suite(suite_name): dependency
            for suite_name, dependency in profile_setting.plugin_suite_setting.items()
        }

        profile_obj = ProfileObject(
            profile_name, profile_setting, engine_provider, suites
        )
        return profile_obj

    def get_profile_setting(self, name) -> ProfileSetting:
        """
        Returns
        -------
        Returns the profile data what stores the profile setting
        """
        if not self.is_profile(name):
            raise ProfileNotFound(name)
        else:
            profile = self.profiles[name]
            return profile

    def is_suite_used_by_profile(self, suite_name: str) -> bool:
        """
        Returns
        -------
        Returns true if a suite identified by suite_name is used by any of the profiles
        """
        for profile in self.profiles.values():
            for suite in profile.plugin_suite_setting.keys():
                if suite == suite_name:
                    return True
        return False

    def is_engine_used_by_profile(self, engine_name: str) -> bool:
        """
        Returns
        -------
        Returns true if engine identified by engine_name is used by any of the profiles
        """
        for profile in self.profiles.values():
            if profile.engine_setting_name == engine_name:
                return True
        return False

    def _load_existing_profile(self):
        """
        Load existing profile on the disk
        """
        files = paths_in_dir(self.workspace, ["toml"])
        for file in files:
            name = get_name(file)
            data = read_toml(file)
            try:
                self.profiles[name] = ProfileSetting(
                    engine_setting_name=data["engine_setting_name"],
                    plugin_suite_setting=data["plugin_suite_setting"],
                )
            except Exception as e:
                delete(file)
                self.logger.error(
                    f"failed to load existing profile {name} with error {e}"
                )

    def _load_default_profile(self):
        """
        If default profile is not added, added it to profile manager
        """
        if not self.is_profile(self._default_name):
            self.add_new_profile(self._default_name, self._default_data)



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-09 11:41:12
# @Last Modified by:   Jason Y. Wu
# @Last Modified time: 2023-08-01 05:31:22

# Standard library imports
from typing import Any, BinaryIO, List, Dict, Callable, Union

# Local imports
from gailbot.configs import watson_config_loader
from gailbot.shared.utils.logger import makelogger
from gailbot.shared.utils.media import MediaHandler
from .codes import WatsonReturnCodes

# Third party imports
from ibm_watson import SpeechToTextV1
from ibm_cloud_sdk_core.authenticators import IAMAuthenticator
from gailbot.shared.exception.transcribeException import (InternetConnectionError,
                                                          WatsonMethodExecutionError)

logger = makelogger("watson_am_model")

WATSON_CONFIG = watson_config_loader()


class WatsonAMInterface:
    """
    Responsible for interacting with the IBM Watson STT service and providing
    methods for interacting with acoustic models.
    """

    def __init__(self, apikey: str, region: str):
        self.apikey = apikey
        self.region = region
        self.media_h = MediaHandler()
        self.connected_to_service = False
        # Parse confs
        self._regions = WATSON_CONFIG.regions_uris
        self._format_to_content_types = WATSON_CONFIG.format_to_content
        self._defaults = WATSON_CONFIG.defaults
        self.max_size_bytes = WATSON_CONFIG.max_file_size_bytes

        if not self._is_api_key_valid(apikey, self._regions[region]):
            raise Exception(f"Apikey {apikey} invalid")
        if not region in self._regions:
            raise Exception(f"Region {region} not in {list(self._regions.keys())}")
        try:
            # Create the stt service and run
            authenticator = IAMAuthenticator(self.apikey)
            self.stt = SpeechToTextV1(authenticator=authenticator)
            self.stt.set_service_url(self._regions[self.region])
        except Exception as e:
            logger.error(e, exc_info=True)
            raise InternetConnectionError()
        else:
            self.connected_to_service = True

    def get_custom_model(self, customization_id: str) -> Union[Dict[str, Any], None]:
        """
        Obtain information regarding a specific custom acoustic model.

        Args:
            customization_id (str): Unique ID of the custom acoustic model.

        Returns:
            (Union[Dict[str,Any],None]):
                Mapping from the following keys to their values:
                customization_id, CREATED, updated,language, dialect,
                versions, owner, name, description, base_model_name, status,
                progress
                None if unsuccessful.
        """
        _, resp = self._execute_watson_method(
            self.stt.get_acoustic_model, [WatsonReturnCodes.OK], [customization_id]
        )
        return resp

    def get_custom_models(self) -> Dict[str, str]:
        """
        Obtain all custom acoustic models

        Returns:
            (Dict[str,str]):
                Mapping from custom acoustic model to the customization id.
        """
        custom_models = dict()
        response = self._execute_watson_method(
            self.stt.list_acoustic_models, [WatsonReturnCodes.OK]
        )
        if response:
            for model in response["customizations"]:
                custom_models[model["name"]] = model["customization_id"]
        return custom_models

    def create_custom_model(
            self, name: str, base_model_name: str, description: str
    ) -> bool:
        """
        Create a new custom acoustic model.

        Args:
            name (str): Name of the model.
            base_model_name (str):
                Name of the base model. Must be a supported base model.
            description (str): Description of the model.

        Returns:
            (bool): True if successful. False otherwise.
        """
        response = self._execute_watson_method(
            self.stt.create_acoustic_model,
            [WatsonReturnCodes.CREATED],
            [name, base_model_name],
            {"description": description},
        )
        return response

    def delete_custom_model(self, customization_id: str) -> bool:
        """
        Delete an existing custom acoustic model.

        Args:
            customization_id (str): Unique ID of the custom acoustic model.

        Returns:
            (bool): True if successful. False otherwise.
        """
        response = self._execute_watson_method(
            self.stt.delete_acoustic_model, [WatsonReturnCodes.OK], [customization_id]
        )
        return response

    def train_custom_model(self, customization_id: str) -> bool:
        """
        Train a custom acoustic model with a previously added resource.

        Args:
            customization_id (str): Unique ID of the custom acoustic model.

        Returns:
            (bool): True if successful. False otherwise.
        """
        response = self._execute_watson_method(
            self.stt.train_acoustic_model, [WatsonReturnCodes.OK], [customization_id]
        )
        return response

    def reset_custom_model(self, customization_id: str) -> bool:
        """
        Reset a custom acoustic model to remove all loaded resources.

        Args:
            customization_id (str): Unique ID of the custom acoustic model.

        Returns:
            (bool): True if successful. False otherwise.
        """
        response = self._execute_watson_method(
            self.stt.reset_acoustic_model, [WatsonReturnCodes.OK], [customization_id]
        )
        return response

    def upgrade_custom_model(
            self, customization_id: str, custom_language_model_id: str = None
    ) -> bool:
        """
        Upgrade the base model of the custom acoustic model to its latest
        version.

        Args:
            customization_id (str): Unique ID of the custom acoustic model.
            custom_language_model_id (str):
                ID of the custom language model (if any) that this acoustic
                model was trained with a custom language model.

        Returns:
            (bool): True if successful. False otherwise.
        """
        response = self._execute_watson_method(
            self.stt.upgrade_acoustic_model,
            [customization_id, custom_language_model_id],
        )
        return response

    def get_custom_audio_resources(
            self, customization_id: str
    ) -> Union[List[Dict], None]:
        """
        List information about all audio resources of the specified custom
        acoustic model.

        Args:
            customization_id (str): Unique ID of the custom acoustic model.

        Returns:
            (Union[List[Dict],None]):
                Each internal list contains the following keys:
                "duration","name","details","status"
                None if unsuccessful.
        """
        response = self._execute_watson_method(self.stt.list_audio, [customization_id])
        if response:
            return response["audio"]

        def get_custom_audio_resource(
                self, customization_id: str, audio_name: str
        ) -> Union[Dict, None]:
            """
            Obtain information about a specific custom acoustic model.

            Args:
                customization_id (str): Unique ID of the custom acoustic model.
                audio_name (str): Name of the specific audio resource.

            Returns:
                (Dict): Contains the following keys:
                        "duration","name","details","status"
                        None if unsuccessful
            """
            response = self._execute_watson_method(
                self.stt.get_audio,
                [WatsonReturnCodes.OK],
                [customization_id, audio_name],
            )
            if response:
                return response

        def add_custom_audio_resource(
                self,
                customization_id: str,
                audio_name: str,
                audio_resource: BinaryIO,
                content_type: str,
        ) -> bool:
            """
            Add an audio resource to the specific custom acoustic model.

            Args:
                customization_id (str): Unique ID of the custom acoustic model.
                audio_name (str): Name of the specific audio resource.
                audio_resource (BinaryIO):
                    Audio resources to be added to the custom model.
                content_type (str):
                    Type of the audio resource. One of:
                    [application/zip,application/gzip,audio/alaw,audio/basic,
                    audio/flac,audio/g729,audio/l16,audio/mp3,audio/mpeg,
                    audio/mulaw,audio/ogg,audio/ogg;codecs=opus,
                    audio/ogg;codecs=vorbis,audio/wav,audio/webm,
                    audio/webm;codecs=opus,audio/webm;codecs=vorbis]

            Returns:
                (bool): True if successful. False otherwise.
            """
            response = self._execute_watson_method(
                self.stt.add_audio,
                [WatsonReturnCodes.CREATED],
                [customization_id, audio_name, audio_resource, content_type],
            )
            return response

    def delete_custom_audio_resource(
            self, customization_id: str, audio_name: str
    ) -> bool:
        """
        Delete the specified resource from the custom audio model.

        Args:
            customization_id (str): Unique ID of the custom acoustic model.
            audio_name (str): Name of the specific audio resource.

        Returns:
            (bool): True if successful. False otherwise.
        """
        response = self._execute_watson_method(
            self.stt.delete_audio,
            [WatsonReturnCodes.OK],
            [customization_id, audio_name],
        )
        return response

    ## PRIVATE ##

    def _is_api_key_valid(self, apikey: str, url: str) -> bool:
        try:
            stt = self._initialize_stt_service(apikey)
            stt.set_service_url(url)
            stt.list_models()
            return True
        except:
            return False

    def _initialize_stt_service(self, apikey: str) -> SpeechToTextV1:
        authenticator = IAMAuthenticator(apikey)
        stt = SpeechToTextV1(authenticator=authenticator)
        return stt

    def _execute_watson_method(
            self,
            method: Callable,
            expected_response_codes: List[WatsonReturnCodes],
            args: List = None,
            kwargs: Dict = None,
    ) -> Union[bool, Any]:
        """
        Execute a watson method only if connected to watson.

        Args:
            method (Callable): Method to execute.
            expected_response_codes (List[WatsonReturnCodes]):
                Watson codes that are expected for a successful response.
            args (List): Arguments to pass to the method.
            kwargs (Dict): Keyword arguments to method

        Returns:
           (Union[bool,Any]):
                result from watson if successful.
        Raises:
            WatsonMethodExecutionError
        """
        if not self.connected_to_service:
            raise InternetConnectionError
        try:
            if not args:
                args = []
            if not kwargs:
                kwargs = {}
            resp = method(*args, **kwargs)
            if any(
                    [
                        resp.get_status_code() == expected
                        for expected in expected_response_codes
                    ]
            ):
                return resp.get_result()
            raise WatsonMethodExecutionError()
        except Exception as e:
            logger.error(f"Exception raised: {e}", exc_info=e)
            raise WatsonMethodExecutionError()



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-01-29 13:26:17
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-02-06 16:08:45
# @Description: a wrapper around Plugin object, PluginComponent will eventually
#  be sent to Pipeline, which accepts executable function stored in component object
import time
from typing import Dict, Any

from gailbot.shared.pipeline import Component, ComponentResult, ComponentState
from gailbot.shared.utils.logger import makelogger
from gailbot.pluginSuiteManager.suite.plugin import Plugin


class PluginResult(ComponentResult):
    state: ComponentState = ComponentState.FAILED
    result: Any = None
    runtime: float = 0


class PluginComponent(Component):
    """
    This is an adapter because the Plugin expects different args as compared
    to transcriptionPipeline components.
    This is needed to so that ComponentResult component is not passed to the user.
    """

    logger = makelogger("pluginSuite")

    def __init__(self, plugin: Plugin):
        super().__init__()
        self.plugin = plugin

    def __repr__(self):
        return str(self.plugin)

    def __call__(
        self,
        dependency_outputs: Dict[str, ComponentResult] = None,
        methods=None,
        *args,
        **kwargs,
    ):
        """
        In addition to dependency outputs, this expects methods which can be
        passed to the individual plugins.
        """
        if not dependency_outputs:
            dependency_outputs = None
        # Extract the actual dependency results
        dep_outputs = {k: v.result for k, v in dependency_outputs.items()}
        # Simply call the plugin and return its results
        start = time.time()
        try:
            result = self.plugin.apply(dep_outputs, methods)
        except Exception as e:
            result = f"Error: {e}"
            self.logger.error(e, exc_info=e)

        elapsed = time.time() - start

        return PluginResult(
            state=(
                ComponentState.SUCCESS
                if self.plugin.is_successful
                else ComponentState.FAILED
            ),
            result=result,
            runtime=elapsed,
        )



# -*- coding: utf-8 -*-
# @Author: Jason Y. Wu
# @Date:   2023-06-28 18:22:18
# @Last Modified by:   Joanne Fan
# @Last Modified time: 2024-05-16 16:29:45
from gailbot.shared.exception.gbException import GBException


class InternetConnectionError(GBException):
    def __init__(self):
        super().__init__()
        self.code = 404
        self.error_msg = " STT Connection Error"


class TranscriptionError(GBException):
    def __init__(self, error: str = None) -> None:
        super().__init__()
        self.code = 500
        self.error = error
        self.error_msg = f"Transcription error: {self.error}"


class APIKeyError(GBException):
    def __init__(self) -> None:
        super().__init__()
        self.code = 508
        self.error_msg = "API key error"


class AudioFileError(GBException):
    def __init__(self) -> None:
        super().__init__()
        self.code = 510
        self.error_msg = "Not a valid audio file"


class ModelCreateError(GBException):
    def __init__(self) -> None:
        super().__init__()
        self.code = 511
        self.error_msg = "Model creation error"


class WatsonMethodExecutionError(GBException):
    def __init__(self) -> None:
        super().__init__()
        self.code = 512
        self.error_msg = "Watson method execution error"

class WatsonModelNameTaken(GBException):
    def __init__(self) -> None:
        super().__init__()
        self.code = 513
        self.error_msg = "Watson custom model name is already taken."

class OutPutError(GBException):
    def __init__(self) -> None:
        super().__init__()
        self.code = 520
        self.error_msg = "Error writing output"


class GetUttResultError(GBException):
    def __init__(self) -> None:
        super().__init__()
        self.code = 521
        self.error_msg = "Failed to get utterance result"



# -*- coding: utf-8 -*-
# @Author: Joanne Fan
# @Date:   2024-02-28 23:11:18
# @Email:
# @Last Modified by:   Joanne Fan
# @Last Modified time: 2024-03-13 15:18:26
# @Descriptions

import pytest
import os.path
from gailbot.shared.exception.serviceException import IncompatibleFileFormat
from gailbot.payload import ConvertFun
from test.test_data import TestData


@pytest.fixture
def converter():
    return ConvertFun()


def run_convert(converter, filepath, tgt_par_dir):
    filename = os.path.basename(filepath)
    no_ext_filename = os.path.splitext(filename)[0]

    tgt_wavpath_list = converter.convert_to_wav(filepath, str(tgt_par_dir))

    if len(tgt_wavpath_list) == 1:
        outfile_name = f"{no_ext_filename}.wav"
        outfile_path = tgt_par_dir.join(outfile_name)
        assert outfile_path.exists()
        assert tgt_wavpath_list[0] == outfile_path
    else:
        for i, tgt_wavpath in enumerate(tgt_wavpath_list):
            outfile_name = f"{no_ext_filename}{i + 1}.wav"
            outfile_path = tgt_par_dir.join(outfile_name)
            assert outfile_path.exists()
            assert tgt_wavpath == outfile_path

    converter.convert_to_wav(filepath, TestData.OUTPUT)


@pytest.mark.parametrize("filename", ['hello.mp3',
                                      'hello.aac',
                                      'hello.mp4',
                                      'hello.mov',
                                      'hello.avi',
                                      'hello.3gp',
                                      'short.mxf'
                                      ])
def test_convert_formats(converter, filename, tmpdir):
    tgt_parent_dir = tmpdir.mkdir("output")
    filepath = os.path.join(TestData.INPUT, filename)
    run_convert(converter, filepath, tgt_parent_dir)


# The file's content is a different audio format than the audio format
# named by the extension. Should raise exception.
@pytest.mark.parametrize("filename", ['hello1aac.mp3'])
def test_convert_mismatched_audios(converter, filename, tmpdir):
    tgt_parent_dir = tmpdir.mkdir("output")
    filepath = os.path.join(TestData.BAD_INPUT, filename)

    with pytest.raises(IncompatibleFileFormat) as e:
        converter.convert_to_wav(filepath, str(tgt_parent_dir))


# The file's content is a different video format than the video format
# named by the extension. Should not raise exception. 
# TODO mxf NOT included in this case.
@pytest.mark.parametrize("filename", ['failvidmov.mp4',
                                      'hello3gp.avi',
                                      'hello1avi.3gp',
                                      'hello1avi.mov'
                                      ])
def test_convert_mismatched_videos(converter, filename, tmpdir):
    tgt_parent_dir = tmpdir.mkdir("output")
    filepath = os.path.join(TestData.BAD_INPUT, filename)
    run_convert(converter, filepath, tgt_parent_dir)


# The file's content is not wav, even though the extension is .wav. Should
# raise exception.
@pytest.mark.parametrize("filename", ['hello1mp3.wav'])
def test_convert_wav_fail(converter, filename, tmpdir):
    tgt_parent_dir = tmpdir.mkdir("output")
    filepath = os.path.join(TestData.BAD_INPUT, filename)

    with pytest.raises(IncompatibleFileFormat) as e:
        converter.convert_to_wav(filepath, str(tgt_parent_dir))


# audio vs video mismatched should raise exception.
@pytest.mark.parametrize("filename", ['hellowav.mov'])
def test_convert_mismatched_vid_audio(converter, filename, tmpdir):
    tgt_parent_dir = tmpdir.mkdir("output")
    filepath = os.path.join(TestData.BAD_INPUT, filename)

    with pytest.raises(IncompatibleFileFormat) as e:
        converter.convert_to_wav(filepath, str(tgt_parent_dir))



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-03-29 17:10:22
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-05-10 13:36:14

from pydantic import BaseModel
from typing import List
from gailbot.setting_interface.formItem import FormItem, FormType
from google.cloud.speech_v1p1beta1.types import SpeechAdaptation
from gailbot.setting_interface.formItem import FormItem


class GoogleSetting(BaseModel):
    engine: str = "google"
    google_api_key: str
    language_code: str = "en-US"
    enable_separate_recognition_per_channel: bool = False
    profanity_filter: bool = False
    enable_word_time_offsets: bool = False
    enable_word_confidence: bool = False
    enable_automatic_punctuation: bool = False
    enable_spoken_punctuation: bool = False
    enable_spoken_emojis: bool = False
    use_enhanced: bool = False
    model: str = "default"
    phrase_sets: List[str] = []
    sample_rate_hertz: int = 16000
    
    

    @staticmethod
    def get_setting_config() -> list[FormItem]:
        return [
            FormItem(type=FormType.Selection, name="engine", selection_items=["google"], default_value="google"),
            FormItem(type=FormType.Selection, name="language_code", selection_items=['en-US', 'ar-KW', 'ru-RU', 'sv-SE', 'lt-LT', 'it-CH', 'en-GH', 'en-NZ', 'tn-Latn-ZA', 'ta-LK', 'ar-EG', 'ar-YE', 'kk-KZ', 'bg-BG', 'ar-PS', 'en-CA', 'es-PY', 'ar-JO', 'es-AR', 'es-DO', 'zh-CN', 'es-CR', 'ar-MA', 'uz-UZ', 'sq-AL', 'mn-MN', 'th-TH', 'hr-HR', 'uk-UA', 'rw-RW', 'pa-Guru-IN', 'bs-BA', 'en-SG', 'en-NG', 'hy-AM', 'lv-LV', 'ss-Latn-ZA', 'fr-CA', 'es-CL', 'en-TZ', 'en-IE', 'jv-ID', 'en-PH', 'bn-IN', 'ar-BH', 'ar-IQ', 'it-IT', 'es-PE', 'si-LK', 'te-IN', 'ml-IN', 'en-AU', 'zu-ZA', 'en-HK', 'bn-BD', 'es-ES', 'sw-KE', 'ar-OM', 'es-VE', 'et-EE', 'gu-IN', 'es-SV', 'ar-SY', 'ko-KR', 'tr-TR', 'es-PR', 'hu-HU', 'ur-IN', 'es-NI', 'fi-FI', 'ta-MY', 'ts-ZA', 'vi-VN', 'mr-IN', 'sl-SI', 'en-ZA', 'kn-IN', 'ur-PK', 'es-CO', 'pt-PT', 'de-AT', 'ar-IL', 'en-KE', 'yue-Hant-HK', 'nl-BE', 'ar-MR', 'mk-MK', 'id-ID', 'pl-PL', 'de-DE', 'es-PA', 'ka-GE', 'en-PK', 'ca-ES', 'hi-IN', 'km-KH', 'es-EC', 'eu-ES', 'iw-IL', 'su-ID', 'am-ET', 'fa-IR', 'ms-MY', 've-ZA', 'nl-NL', 'el-GR', 'es-MX', 'lo-LA', 'es-BO', 'en-GB', 'es-US', 'no-NO', 'fil-PH', 'en-IN', 'ar-DZ', 'ar-LB', 'zh-TW', 'gl-ES', 'es-GT', '(SouthAfrica)', 'ro-RO', 'ar-QA', 'ta-SG', 'ar-SA', 'fr-BE', 'ta-IN', 'cs-CZ', 'es-HN', 'my-MM', 'ja-JP', 'af-ZA', 'is-IS', 'ar-TN', 'az-AZ', 'fr-FR', 'pt-BR', 'sr-RS', 'sk-SK', 'da-DK', 'de-CH', 'ne-NP', 'es-UY', 'fr-CH', 'ar-AE', 'xh-ZA', 'sw-TZ'], default_value="en-US"),

            FormItem(type=FormType.File, name="google_api_key"),
            
            # booleans 
            FormItem(type=FormType.OnOff, name="enable_separate_recognition_per_channel", default_value=False),
            FormItem(type=FormType.OnOff, name="profanity_filter", default_value=False),
            FormItem(type=FormType.OnOff, name="enable_word_time_offsets", default_value=False),
            FormItem(type=FormType.OnOff, name="enable_word_confidence", default_value=False),
            FormItem(type=FormType.OnOff, name="enable_automatic_punctuation", default_value=False),
            FormItem(type=FormType.OnOff, name="enable_spoken_punctuation", default_value=False),
            FormItem(type=FormType.OnOff, name="enable_spoken_emojis", default_value=False),
            FormItem(type=FormType.OnOff, name="use_enhanced", default_value=False),

            # multi select
            FormItem(type=FormType.MultiSelect, name="phrase_sets", selection_items=[], default_value=None),

            # Selection 
            FormItem(type=FormType.Selection, name="model", selection_items=["latest_long", "latest_short", "command_and_search", "phone_call", "video", "default", "medical_conversation", "medical_dictation"], default_value="default"), 
            FormItem(type=FormType.Selection, name="sample_rate_hertz", selection_items=[8000, 16000, 22050, 32000, 44100, 48000], default_value=16000),
        ]  



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-01-23 08:04:39
# @Last Modified by:   Vivian Li 
# @Last Modified time: 2024-03-03 15:39:24
from gailbot.shared.exception.gbException import GBException


class RemoveDefaultEngine(GBException):
    def __init__(self):
        self.code = 14
        self.error_msg = "Default Engine cannot be removed"


class RemoveDefaultProfile(GBException):
    def __init__(self):
        self.code = 14
        self.error_msg = "Default Profile cannot be removed"


class RemoveOfficialPlugin(GBException):
    def __init__(self):
        self.code = 32
        self.error_msg = "Official Plugin Suite cannot be removed"


class NotFound(GBException):
    def __init__(self, name: str):
        super().__init__()
        self.name = name
        self.error_msg = f"{self.name} not found."


class AddError(GBException):
    def __init__(self, name: str, cause: Exception):
        super().__init__()
        self.name = name
        self.cause = cause
        self.error_msg = f"Fail to add {self.name} due to {self.cause.__cause__}"


class Duplicate(GBException):
    def __init__(self, name):
        super().__init__()
        self.code = 13
        self.name = name
        self.error_msg = f"{self.name} already exists"


class DuplicatePlugin(Duplicate):
    def __init__(self, name):
        super().__init__(name)
        self.code = 44  # undocumented


class DuplicateEngine(Duplicate):
    def __init__(self, name):
        super().__init__(name)


class DuplicateProfile(Duplicate):
    def __init__(self, name):
        super().__init__(name)


# TODO: improve naming?
class RemoveInUseError(GBException):
    def __init__(self, name: str):
        super().__init__()
        self.name = name
        self.error_msg = f"Fail to remove {self.name} that is being used."


class SourceAddError(AddError):
    def __init__(self, name: str, cause: Exception):
        super().__init__(name, cause)
        self.code = 7


class ProfileAddError(AddError):
    def __init__(self, name: str, cause: Exception):
        super().__init__(name, cause)
        self.code = 16


class EngineAddError(AddError):
    def __init__(self, name: str, cause: Exception):
        super().__init__(name, cause)
        self.code = 23


class SourceNotFound(NotFound):
    def __init__(self, name):
        self.code = 4
        super().__init__(name)


class ProfileNotFound(NotFound):
    def __init__(self, name):
        self.code = 19
        super().__init__(name)


class EngineNotFound(NotFound):
    def __init__(self, name):
        self.code = 26
        super().__init__(name)


class PluginSuiteNotFound(NotFound):
    def __init__(self, name):
        self.code = 40  # not documented
        super().__init__(name)


class FailPluginSuiteRegister(GBException):
    def __init__(self, path: str, cause: str):
        super().__init__()
        self.cause = cause
        self.code = 35
        self.path = path
        self.error_msg = (
            f"Fail to register plugin suite at {path} due to : {self.cause}"
        )

    def __str__(self):
        return self.cause


class FailPluginSuiteReload(GBException):
    def __init__(self, name: str, cause: Exception):
        super().__init__()
        self.name = name
        self.code = 41  # undocumented
        self.cause = cause
        self.error_msg = (
            f"Fail to reload plugin suite {self.name} due to : {self.cause}"
        )


class FailReportError(GBException):
    def __init__(self, name: str, cause: Exception):
        super().__init__()
        self.name = name
        self.code = 45  # undocumented
        self.cause = cause
        self.error_msg = f"Fail to report suite registration error for {self.name} due to : {self.cause}"


class IncompatibleConverter(GBException):
    def __init__(self, source_name: str, converter: str):
        super().__init__()
        self.code = 100  # undocumented
        self.error_msg = f"{converter} is not compatible with {source_name}"


class IncompatibleFileFormat(GBException):
    def __init__(self, filepath: str, extension: str):
        super().__init__()
        self.code = 101  # undocumented
        self.error_msg = f"Content of {filepath} is not compatible with {extension}"



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-02-24 10:12:13
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-05-10 13:36:52
# @Description : convert a single audio file to a singleton list of one
#                audio payload
from datetime import datetime
from typing import List
from .converterFun import ConvertFun
from ..payloadObject import PayloadObject
from ..payloadObject.untranscribedDirPayload import untranscribedDirPayload
from ...profileManager import ProfileObject
from ...shared.utils.general import get_extension, get_name
from ...sourceManager.watcher import Watcher
from ...workspace.directory_structure import TemporaryFolder
from .converter import Converter, ConverterType


class MediaConverter(Converter):
    accepted_forms = list(ConvertFun.convert_funs.keys())
    converter_type = ConverterType.SingleMedia

    @staticmethod
    def is_accepted_form(path: str) -> bool:
        """
        Return true if path is an audio file with accepted format
        """
        format = get_extension(path)
        return format.lower() in MediaConverter.accepted_forms

    @staticmethod
    def convert(
        path: str, output_dir: str, profile: ProfileObject, watchers: List[Watcher]
    ) -> List[PayloadObject]:
        """
        convert path to a list of one AudioPayloadObject

        Parameters
        ----------
        path
        output_dir
        profile
        watchers

        Returns
        -------
        A list of one audio payload object
        """

        name = get_name(path)
        time = datetime.now().strftime("%m%d_%H_%M_%S_%f")
        temp_path = TemporaryFolder(f"{name}_{time}")
        audio_path: List[str] = ConvertFun.convert_to_wav(path, temp_path.data_copy)

        return [
            untranscribedDirPayload(
                original_source=path,
                data_files=audio_path,
                temp_dir_struct=temp_path,
                output_path=output_dir,
                profile=profile,
                watchers=watchers,
            )
        ]



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2021-12-02 13:13:08
# @Last Modified by:   Jason Y. Wu
# @Last Modified time: 2023-08-01 05:31:59
"""
    The classes in this script model the RecognitionResult object as per
    the documentation for IBM Watson STT service:
    https://cloud.ibm.com/docs/speech-to-text?topic=speech-to-text-basic-response
"""
# Standard library imports
from typing import Dict, Any, List
from gailbot.shared.utils.logger import makelogger
import itertools

# Local imports
# Third party imports

logger = makelogger("recognition_result")


class Alternative:
    """
    Models the smallest unit of a transcription result.
    """

    def __init__(self, data: Dict) -> None:
        self.items = {
            "transcript": None,
            "confidence": None,
            "timestamps": None,
            "word_confidence": None,
        }
        self.configured = self._parse_data(data)

    def is_configured(self) -> bool:
        """
        Return True if the data was successfully parsed. False otherwise.
        """
        return self.configured

    def get(self, key: str) -> Any:
        """
        Obtain the value associated with the specified key.
        None if key not found.
        """
        return self.items[key]

    def _parse_data(self, data: Dict) -> bool:
        """
        Parse the input data dictionary.

        Args:
            data (Dict)

        Returns:
            (bool): True if parsed successfully. False otherwise.
        """
        keys = ["timestamps", "word_confidence", "transcript", "confidence"]
        for key in keys:
            if key in data:
                self.items[key] = data[key]
        return all([k in data for k in keys])


class SpeakerLabel:
    """
    Models a SpeakerLabel data item returned from the IBM Watson STT service.
    """

    def __init__(self, data: Dict) -> None:
        self.items = {"from": None, "to": None, "speaker": None, "confidence": None}
        self.configured = self._parse_data(data)

    def is_configured(self) -> bool:
        """
        Return True if the data was successfully parsed. False otherwise.
        """
        return self.configured

    def get(self, key: str) -> Any:
        """
        Obtain the value associated with the specified key.
        None if key not found.
        """
        return self.items[key]

    def _parse_data(self, data: Dict) -> bool:
        """
        Parse the input data dictionary.

        Args:
            data (Dict)

        Returns:
            (bool): True if parsed successfully. False otherwise.
        """
        for key in self.items.keys():
            if key in data:
                self.items[key] = data[key]
        return all([k in data for k in self.items.keys()])


class Result:
    """
    Models a 'Result' object that is returned from the IBM STT service.
    """

    def __init__(self, data: Dict) -> None:
        self.items = {
            "keywords_result": list(),
            "word_alternatives": list(),
            "alternatives": list(),
            "final": None,
        }
        self._parse_data(data)

    def is_configured(self) -> bool:
        """
        Return True if the data was successfully parsed. False otherwise.
        """
        return self.items and len(self.items) != 0

    def get(self, key: str) -> Any:
        """
        Obtain the value associated with the specified key.
        None if key not found.
        """
        return self.items[key]

    def _parse_data(self, data: Dict):
        """
        Parse the input data dictionary.

        Args:
            data (Dict)
        """
        results = data["results"]
        if "final" in results:
            self.items["final"] = results["final"]
        if "keywords_result" in results.keys():
            self.items["keywords_result"] = results["keywords_result"]
        if "word_alternatives" in results.keys():
            self.items["word_alternatives"] = results["word_alternatives"]
        if "alternatives" in results.keys():
            self.items["alternatives"] = [
                Alternative(alt) for alt in results["alternatives"]
            ]


class RecognitionResult:
    """
    Main class that models an entire RecognitionResult object that is returned
    from the IBM Watson STT service.
    """

    def __init__(self, watson_data: Dict) -> None:
        self.items = {"result_index": None, "results": [], "speaker_labels": []}
        self.configured = self._parse_data(watson_data)

    def is_configured(self) -> bool:
        """
        Return True if the data was successfully parsed. False otherwise.
        """
        return self.configured

    def get_result_index(self) -> int | None:
        """
        Obtain the index / result number returned by the IBM STT service.

        Returns:
            (int): Result index.
        """
        return self.items["result_index"]

    def num_speaker_labels(self) -> int | None:
        """
        Obtain the number of SpeakerLabel objects received by the service.
        Note that this does not relate to the number of speakers detected by
        the service.

        Returns:
            (int): Number of SpeakerLabel objects received.
        """
        if not self.items["speaker_labels"]:
            return None
        return len(self.items["speaker_labels"])

    def num_results(self) -> int | None:
        """
        Obtain the number of individual results obtained by the IBM STT service.
        """
        return len(self.items["results"])

    def get_speaker_labels(self) -> List[Dict[str, str]]:
        """
        Obtain a list of dictionaries that model SpeakerLabel objects.

        Returns:
            (List[Dict[str,str]]):
                List of dictionaries modeling SpeakerLabel objects.
                Each dictionary has the keys: 'from', 'to', 'speaker',
                'confidence'.
        """
        labels = list()
        if not self.items or not self.items["speaker_labels"]:
            return labels

        for label in self.items["speaker_labels"]:
            labels.append(
                {
                    "start_time": label.get("from"),
                    "end_time": label.get("to"),
                    "speaker": label.get("speaker"),
                    "label_confidence": label.get("confidence"),
                }
            )
        return labels

    def get_keywords_results(self, only_final: bool = True) -> List[Any]:
        """
        Obtain a list of 'keyword_result' objects received from the IBM
        STT service.

        Args:
            only_final (bool):
                True to obtain 'keyword_result' only from results that are
                considered final and not interim. False to include all results,
                both final and interim.

        Returns:
            (List[Any]): List of 'keyword_result' objects.
        """
        return self._aggregate(self.items["results"], "keywords_result", only_final)

    def get_word_alternatives(self, only_final: bool = True) -> Any:
        """
        Obtain a list of 'word_alternative' objects received from the IBM
        STT service.

        Args:
            only_final (bool):
                True to obtain 'word_alternative' only from results that are
                considered final and not interim. False to include all results,
                both final and interim.

        Returns:
            (List[Any]): List of 'word_alternative' objects.
        """
        return self._aggregate(self.items["results"], "word_alternatives", only_final)

    def get_transcript_from_alternatives(self, only_final: bool = True) -> List[str]:
        """
        Obtain a list containing all 'transcript' items from all Alternative
        received as part of the IBM STT result.

        Args:
            only_final (bool):
                True to obtain items only from results that are
                considered final and not interim. False to include all results,
                both final and interim.

        Returns:
            (List[str]):
                List of 'transcript' items for all Alternative objects.
        """
        alternatives = self._aggregate(
            self.items["results"], "alternatives", only_final
        )[0]
        return [alternative.get("transcript") for alternative in alternatives]

    def get_transcript_confidences_from_alternatives(
            self, only_final: bool = True
    ) -> List[float]:
        """
        Obtain a list containing all 'transcript_confidence' objects from all
        Alternative received as part of the IBM STT result.

        Args:
            only_final (bool):
                True to obtain items only from results that are
                considered final and not interim. False to include all results,
                both final and interim.

        Returns:
            (List[float]):
                List containing confidence for each transcript obtained.
        """
        alternatives = self._aggregate(
            self.items["results"], "alternatives", only_final
        )[0]

        return [alternative.get("confidence") for alternative in alternatives]

    def get_timestamps_from_alternatives(
            self, only_final: bool = True
    ) -> List[List[List[Any]]]:
        """
        Obtain a list containing all 'timestamps' objects from all
        Alternative received as part of the IBM STT result.

        Args:
            only_final (bool):
                True to obtain items only from results that are
                considered final and not interim. False to include all results,
                both final and interim.

        Returns:
            (List[List[List[Any]]]):
                One main list continaing lists, with each inner most list
                representing one result of the form:
                ['text', 'start time', 'end time']
        """
        alternatives = self._aggregate(
            self.items["results"], "alternatives", only_final
        )
        alternatives = list(itertools.chain(*alternatives))
        timestamps = [alternative.get("timestamps") for alternative in alternatives]
        return [timestamp for timestamp in timestamps if timestamp != None]

    def get_word_confidences_from_alternatives(self, only_final: bool = True) -> List:
        """
        Obtain a list containing all 'word_confidence' objects from all
        Alternative received as part of the IBM STT result.

        Args:
            only_final (bool):
                True to obtain items only from results that are
                considered final and not interim. False to include all results,
                both final and interim.

        Returns:
            (List[List[List[Any]]]):
                One main list continaing lists, with each inner most list
                representing one result of the form:
                ['text', 'confidence']
        """
        alternatives = self._aggregate(
            self.items["results"], "alternatives", only_final
        )[0]
        return [alternative.get("word_confidence") for alternative in alternatives]


    def _parse_data(self, watson_data: Dict) -> bool:
        """
        Parse a single RecognitionResult dictionary obtained from the IBM
        STT service.

        Args:
            (Dict): Dictionary representing a RecognitionResult

        Returns:
            (bool): True if p[arsed successfully. False otherwise.
        """
        # Level 1
        try:
            if "result_index" in watson_data.keys():
                self.items["result_index"] = watson_data["result_index"]
            if "speaker_labels" in watson_data.keys():
                labels = list()
                for label in watson_data["speaker_labels"]:
                    labels.append(SpeakerLabel(label))
                self.items["speaker_labels"] = labels
            # Level 2 (In results)
            if "results" in watson_data.keys():
                results_array = list()
                for results in watson_data["results"]:
                    results_array.append(Result({"results": results}))
                self.items["results"] = results_array
            return True
        except Exception as e:
            logger.error(e, exc_info=e)

    def _aggregate(
            self, results: List[Result], aggregation_key: str, only_final: bool
    ) -> List[Any]:
        """
        Aggregate the results from a list of Result objects, based on the
        aggregation key. If only_final = True, only use the results of
        Result objects considered final.
        """
        response = list()
        for item in results:
            if only_final:
                if item.get("final"):
                    response.append(item.get(aggregation_key))
            else:
                response.append(item.get(aggregation_key))
        return response



# -*- coding: utf-8 -*-
# @Author: Jason Y. Wu
# @Date:   2023-07-31 16:21:27
# @Last Modified by:   Hannah Shader
# @Last Modified time: 2023-11-20 12:47:28
from abc import ABC, abstractmethod

from typing import Dict

from pydantic import BaseModel

from gailbot.engineManager.engine.engine import Engine


class EngineProvider(ABC):
    """
    A factory class to provide a engine based on the engine setting
    """
    engine: str
    name: str
    data: BaseModel

    @staticmethod
    @abstractmethod
    def load(name: str, data: Dict[str, str]):
        raise NotImplementedError()

    @abstractmethod
    def make_engine(self) -> Engine:
        raise NotImplementedError()

    @abstractmethod
    def is_cpu_intensive(self) -> bool:
        raise NotImplementedError()

    def engine_data(self) -> Dict[str, str]:
        return self.data.model_dump()



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-03-30 11:57:13
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-05-10 13:37:15
import pytest
from gailbot import GailBot, ProfileSetting
from gailbot.sourceManager.watcher import Watcher
from test.test_data import TestData
from copy import deepcopy


class TestWatcher(Watcher):
    def watch(self, progress: str):
        print(progress)


GOOGLE_API = "/Users/yike/Desktop/gbtest/input/google-api.json"
WHISPERX_ENGINE = "whisperx engine pipeline test"
WHISPERX_PROFILE = "whisperx profile pipeline test"

GOOGLE_ENGINE = "google engine pipeline test"
GOOGLE_PROFILE = "google profile pipeline test"

WHISPER_ENGINE = "whisper engine pipeline test"
WHISPER_PROFILE = "whisper profile pipeline test"

WATSON_ENGINE = "watson engine pipeline test"
WATSON_PROFILE = "watson profile pipeline test"

PLUGIN_SETTING = {"HiLabSuite": ["ChatPlugin", "TextPlugin", "CSVPlugin", "XmlPlugin"]}


@pytest.fixture(scope="module", autouse=True)
def gb():
    gailbot = GailBot()
    # add whisperX profile
    if not gailbot.is_engine(WHISPERX_ENGINE):
        gailbot.add_engine(WHISPERX_ENGINE, {"engine": "whisperX", "language": "en"})
    if not gailbot.is_profile(WHISPERX_PROFILE):
        gailbot.create_profile(
            WHISPERX_PROFILE,
            ProfileSetting(
                engine_setting_name=WHISPERX_ENGINE,
                plugin_suite_setting=deepcopy(PLUGIN_SETTING),
            ),
        )

    # add whisper profile
    whisper_engine_setting = {
        "engine": "whisper",
        "language": "English",
        "speaker_detect": False,
    }
    if not gailbot.is_engine(WHISPER_ENGINE):
        gailbot.add_engine(name=WHISPER_ENGINE, setting=whisper_engine_setting)
    if not gailbot.is_profile(WHISPER_PROFILE):
        gailbot.create_profile(
            name=WHISPER_PROFILE,
            setting=ProfileSetting(
                engine_setting_name=WHISPER_ENGINE,
                plugin_suite_setting=deepcopy(PLUGIN_SETTING),
            ),
        )

    # add google profile
    google_engine_setting = {"engine": "google", "google_api_key": GOOGLE_API}
    if not gailbot.is_engine(GOOGLE_ENGINE):
        assert gailbot.add_engine(name=GOOGLE_ENGINE, setting=google_engine_setting)
    if not gailbot.is_profile(GOOGLE_PROFILE):
        gailbot.create_profile(
            name=GOOGLE_PROFILE,
            setting=ProfileSetting(
                engine_setting_name=GOOGLE_ENGINE,
                plugin_suite_setting=deepcopy(PLUGIN_SETTING),
            ),
        )

    # add watson profile
    WATSON_API_KEY = "MSgOPTS9CvbADe49nEg4wm8_gxeRuf4FGUmlHS9QqAw3"
    WATSON_REGION = "dallas"
    WATSON_BASE_LANG_MODEL = "en-US_NarrowbandModel"
    watson_engine_setting = {
        "engine": "watson",
        "apikey": WATSON_API_KEY,
        "region": WATSON_REGION,
        "base_model": WATSON_BASE_LANG_MODEL,
    }

    if not gailbot.is_engine(WATSON_ENGINE):
        gailbot.add_engine(name=WATSON_ENGINE, setting=watson_engine_setting)
    if not gailbot.is_profile(WATSON_PROFILE):
        gailbot.create_profile(
            name=WATSON_PROFILE,
            setting=ProfileSetting(
                engine_setting_name=WATSON_ENGINE,
                plugin_suite_setting=deepcopy(PLUGIN_SETTING),
            ),
        )

    return gailbot


@pytest.mark.parametrize(
    "profile", [WATSON_PROFILE, WHISPER_PROFILE, WHISPERX_PROFILE, GOOGLE_PROFILE]
)
def test_hello(gb: GailBot, profile: str):
    ids = [gb.add_source(TestData.hello, TestData.OUTPUT) for _ in range(4)]
    for source_id in ids:
        gb.apply_profile_to_source(source_id, profile)
        gb.add_progress_watcher(source_id, TestWatcher())
    result = gb.transcribe()
    assert result.success_output and not (result.failure or result.invalid)


@pytest.mark.parametrize(
    "profile", [WATSON_PROFILE, WHISPER_PROFILE, WHISPERX_PROFILE, GOOGLE_PROFILE]
)
def test_test2ab(gb: GailBot, profile: str):
    ids = [gb.add_source(TestData.test2ab, TestData.OUTPUT) for _ in range(2)]
    for source_id in ids:
        gb.apply_profile_to_source(source_id, profile)
        gb.add_progress_watcher(source_id, TestWatcher())
    result = gb.transcribe()
    assert result.success_output and not (result.failure or result.invalid)


@pytest.mark.parametrize(
    "profile", [WATSON_PROFILE, WHISPER_PROFILE, WHISPERX_PROFILE, GOOGLE_PROFILE]
)
def test_callhome_small(gb: GailBot, profile: str):
    ids = [gb.add_source(TestData.callhome_small, TestData.OUTPUT) for _ in range(2)]
    for source_id in ids:
        gb.apply_profile_to_source(source_id, profile)
        gb.add_progress_watcher(source_id, TestWatcher())
    result = gb.transcribe()
    assert result.success_output and not (result.failure or result.invalid)


def test_previously_transcribed(gb: GailBot):
    ids = [
        gb.add_source(TestData.test2ab_transcribed, TestData.OUTPUT),
        gb.add_source(TestData.test0638_transcribed, TestData.OUTPUT),
    ]
    for source_id in ids:
        gb.add_progress_watcher(source_id, TestWatcher())
    result = gb.transcribe()
    assert result.success_output and not (result.failure or result.invalid)


@pytest.mark.parametrize("path", [TestData.test_mp3])
def test_mp3(gb: GailBot, path):
    ids = [
        gb.add_source(path, TestData.OUTPUT),
        gb.add_source(path, TestData.OUTPUT),
    ]
    for source_id in ids:
        gb.add_progress_watcher(source_id, TestWatcher())
    result = gb.transcribe()
    assert result.success_output and not (result.failure or result.invalid)


def test_whisper_whisperX(gb: GailBot):
    ids = [
        gb.add_source(TestData.hello, TestData.OUTPUT),
        gb.add_source(TestData.hello, TestData.OUTPUT),
    ]

    for source_id in ids:
        gb.add_progress_watcher(source_id, TestWatcher())

    gb.apply_profile_to_source(ids[0], WHISPER_PROFILE)
    gb.apply_profile_to_source(ids[1], WHISPERX_PROFILE)
    result = gb.transcribe()
    assert result.success_output
    assert not (result.failure or result.invalid)
    print(result)



import os
import unittest

from gailbot.pluginSuiteManager.pluginSuiteManager import PluginSuiteManager
from gailbot.pluginSuiteManager.suiteLoader import PluginURLSuiteLoader
from gailbot.shared.utils.general import delete, make_dir
from test.suiteManager import SUITE_TEST_SRC_PATH, SUITE_TEST_WORKSPACE_PATH


class MyTestCase(unittest.TestCase):
    plugin_suite_manager = PluginSuiteManager(SUITE_TEST_WORKSPACE_PATH)
    src_root = SUITE_TEST_SRC_PATH
    suite_ws = os.path.join(SUITE_TEST_WORKSPACE_PATH, "suites")
    download_ws = os.path.join(SUITE_TEST_WORKSPACE_PATH, "download")
    f = open(os.path.join(SUITE_TEST_WORKSPACE_PATH, "log.txt"), "w+")

    def test_load_invalid(self):
        loader = PluginURLSuiteLoader(self.download_ws, self.suite_ws)
        assert not loader.load("invalid-url", self.f)

    def test_zip_url(self):
        url = "https://gailbot-plugin-suite-official.s3.us-east-2.amazonaws.com/HiLabSuite.zip"
        delete(self.download_ws)
        delete(self.suite_ws)
        make_dir(self.download_ws)
        make_dir(self.suite_ws)
        assert self.plugin_suite_manager.register_suite(url)

    def test_bucket(self):
        bucket = "gailbot-plugin-suite-official"
        delete(self.download_ws)
        delete(self.suite_ws)
        make_dir(self.download_ws)
        make_dir(self.suite_ws)
        assert self.plugin_suite_manager.register_suite(bucket)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-12 13:39:44
# @Last Modified by:   Muhammad Umair

import shutil
from typing import List
from tqdm.auto import tqdm
import requests
from zipfile import ZipFile
import os
import socket
from .logger import makelogger
from gailbot.pluginSuiteManager.S3BucketManager import S3BucketManager
from urllib.parse import urlparse
from typing import Optional, List

logger = makelogger("download")



def download_single_url(
    url: str,
    download_path: str,
    extract_root: str,
    s3_client,
    unzip: bool = True,
    true_extract: bool = False
) -> Optional[str]:
    """
    Download one S3 URL to download_path, unzip under extract_root, and
    return the extract directory (or extract_root if true_extract=True), 
    or the zip path if unzip=False, or None on error.
    """
    try:
        parsed = urlparse(url)
        bucket = parsed.netloc.split(".")[0]
        key = parsed.path.lstrip("/")
        name = os.path.splitext(os.path.basename(key))[0]

        local_zip = os.path.join(download_path, f"{name}.zip")
        os.makedirs(os.path.dirname(local_zip), exist_ok=True)

        logger.info("Downloading s3://%s/%s → %s", bucket, key, local_zip)
        s3_client.download_file(bucket, key, local_zip)
    except Exception as e:
        logger.error("Failed to download %s: %s", url, e, exc_info=True)
        return None

    if not unzip:
        return local_zip

    try:
        # always unzip into a temp subfolder first
        subfolder = os.path.join(extract_root, name)
        if os.path.exists(subfolder):
            shutil.rmtree(subfolder)
        os.makedirs(subfolder, exist_ok=True)

        with ZipFile(local_zip, "r") as zf:
            zf.extractall(subfolder)

        if true_extract:
            # move everything from subfolder → extract_root
            for item in os.listdir(subfolder):
                src = os.path.join(subfolder, item)
                dst = os.path.join(extract_root, item)
                # if dst already exists, overwrite or handle as needed
                if os.path.exists(dst):
                    if os.path.isdir(dst):
                        shutil.rmtree(dst)
                    else:
                        os.remove(dst)
                shutil.move(src, dst)
            shutil.rmtree(subfolder)
            logger.info("Flattened contents of %s into %s", name, extract_root)
            return extract_root
        else:
            logger.info("Extracted %s → %s", local_zip, subfolder)
            return subfolder

    except Exception as e:
        logger.error("Failed to extract %s: %s", local_zip, e, exc_info=True)
        return None

def download_from_urls(
    urls: List[str],
    download_dir: str,
    unzip: bool = True,
) -> List[str]:
    """
    Download a list of S3 URLs and return the list of extracted paths.
    """
    dataset_download_path = os.path.join(download_dir, "download")
    dataset_extract_path = download_dir

    # prepare dirs
    if os.path.isdir(dataset_download_path):
        shutil.rmtree(dataset_download_path)
    os.makedirs(dataset_download_path, exist_ok=True)
    os.makedirs(dataset_extract_path, exist_ok=True)

    s3 = S3BucketManager.get_instance().get_s3()

    extracted = []
    for url in urls:
        logger.info("Processing URL: %s", url)
        path = download_single_url(
            url,
            download_path=dataset_download_path,
            extract_root=dataset_extract_path,
            s3_client=s3,
            unzip=unzip
        )
        if path:
            extracted.append(path)

    return extracted


def is_internet_connected() -> bool:
    """
    True if connected to the internet, false otherwise
    """
    try:
        # connect to the host -- tells us if the host is actually
        # reachable
        sock = socket.create_connection(("www.google.com", 80))
        if sock is not None:
            sock.close()
        return True
    except OSError:
        pass
    return False



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-01-29 13:26:17
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-02-06 15:50:33
# @Implement difference plugin suite loader classes to load plugin suite
# from different kind of sources
from .directoryloader import PluginDirectorySuiteLoader
from .pluginUrlLoader import PluginURLSuiteLoader
from .pluginSuiteLoader import PluginSuiteLoader



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-02-24 10:09:30
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-02-25 17:10:13
# @Description : Implement a converter classes that will convert a source
#                to a list of payloads
from .mediaConverter import MediaConverter
from .conversationDirConverter import ConversationDirConverter
from .mixedDirConverter import MixedDirectoryConverter
from .transcribedDirConverter import TranscribedDirConverter
from .converter import Converter, ConverterType



# -*- coding: utf-8 -*-
# @Author: Jason Y. Wu
# @Date:   2023-07-17 19:21:51
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-02 17:48:10

import os
from typing import List, Dict

from gailbot.shared.utils.general import (
    write_csv,
    read_csv,
    write_json,
    get_name,
    make_dir,
    read_json,
    paths_in_dir,
    is_directory,
    is_file,
)
from gailbot.shared.utils.logger import makelogger
from gailbot.configs import service_config_loader
from .resultInterface import ResultInterface
from gailbot.engineManager.engine.engine import UttDict

SERVICE_CONFIG = service_config_loader()
logger = makelogger("transcribe_result")

"""
TODO: 
use .pickle to store temporary file to save memory
"""


class SttTranscribeResult(ResultInterface):
    """
    Defines a class containing the utterance results of a transcription
    """

    def __init__(self, workspace: str, data: Dict[str, List[UttDict]] = None) -> None:
        super().__init__(workspace, data)
        self.workspace = os.path.join(
            workspace, SERVICE_CONFIG.directory_name.temp_result
        )
        make_dir(self.workspace, overwrite=True)
        self.max_size = 1000  # TODO: move to toml
        self.data = data
        self.filenames: Dict[str, str] = dict()
        self.saved_to_disk: bool = False

    def save_data(self, data: Dict[str, List[UttDict]]) -> bool:
        """
        Saves the given data to the output directory

        Args:
            data: Dict[str, List[UttDict]]: data to save

        Returns:
            bool: True if successfully saved, false if not
        """
        if len(data) == 0:
            logger.error("the result is empty")
            return False
        try:
            for name, result in data.items():
                path = os.path.join(self.workspace, f"{name}.json")
                write_json(path, result)
                if name not in self.filenames:
                    self.filenames[name] = path
            self.saved_to_disk = True
            return True
        except Exception as e:
            logger.error(e, exc_info=e)
            return False

    def output(self, path) -> bool:
        """
        Outputs the result to the output directory

        Args:
            path: output directory path

        Returns:
            bool: True if successfully outputted, false if not
        """
        try:
            if self.saved_to_disk:
                data = self.get_data()
            else:
                data = self.data
            for name, result in data.items():
                write_csv(os.path.join(path, name + ".csv"), result)
            return True
        except Exception as e:
            logger.error(e, exc_info=e)
        return False

    def get_data(self) -> Dict[str, List[UttDict]]:
        """
        Accesses and returns the data of the current transcription result

        Returns:
            Data in the form Dict[str, List[UttDict]]
        """
        if self.saved_to_disk:
            files = paths_in_dir(self.workspace, ["json"])
            res = dict()
            for file in files:
                res[get_name(file)] = read_json(file)
            return res
        else:
            return self.data

    def get_one_file_data(self, name: str) -> Dict[str, List[UttDict]]:
        """
        Accesses and return the data of one file

        Args:
            name: the filename of one file

        Return: a dictionary that contains the utterance data, the key of the
                dictionary will be the file name, and the value will be the
                list of utterance dictionary that stores the utterance data
        """
        res = dict()
        path = self.filenames[name]
        res[name] = read_json(path)
        return res

    def load_result(self, path: str) -> bool:
        """
        Loads the transcription result

        Returns:
            Result in the form Dict[str, List[UttDict]] or false if not
            successfully accessed
        """
        if is_file(path):
            return self._read_from_file(path)
        elif is_directory(path):
            return self._read_from_dir(path)
        else:
            return False

    def _read_from_dir(self, path) -> bool:
        """
        Processes the result of a directory input

        Args:
            path: path to the directory to process

        Returns:
            Dict[str, List[UttDict]]: dictionary containing the processed input
        """
        try:
            files = paths_in_dir(path, ["csv"])
            res = dict()
            for file in files:
                res[get_name(file)] = read_csv(file)
            if res:
                assert self.save_data(res)
            return True
        except Exception as e:
            logger.error(e, exc_info=e)
            return False

    def _read_from_file(self, path: str) -> bool:
        """
        Processes the result of a file input

        Args:
            path: path to the file to process

        Returns:
            Dict[str, List[UttDict]]: dictionary containing the processed input
        """
        try:
            res = {get_name(path): read_csv(path)}
            assert self.save_data(res)
            return True
        except Exception as e:
            logger.error(e, exc_info=e)
            return False



import os.path
import unittest

from gailbot import GailBot, ProfileSetting


class TestGailbotSuiteFunction(unittest.TestCase):
    under_test = GailBot()
    output = "/Users/yike/Desktop/gbtest/output"
    source_root = "/Users/yike/Desktop/gbtest/input"
    small_source = os.path.join(source_root, "hello2.wav")
    test_2ab = os.path.join(source_root, "test2ab")

    def test_existing_suite(self):
        print(self.under_test.available_suites())
        print(self.under_test.get_selectable_plugins("HiLabSuite"))

    def test_all_plugin(self):
        all_selected_plugin = self.under_test.get_selectable_plugins("HiLabSuite")
        profile = ProfileSetting(
            engine_setting_name="Default",
            plugin_suite_setting={"HiLabSuite": all_selected_plugin}
        )
        self.plugin_suite_transcription_test(profile)

    def test_partial_plugin(self):
        selected_plugin = ['GapPlugin', 'PausePlugin', 'OverlapPlugin', 'TextPlugin', 'XmlPlugin', 'ChatPlugin']
        profile = ProfileSetting(
            engine_setting_name="Default",
            plugin_suite_setting={"HiLabSuite": selected_plugin}
        )
        self.plugin_suite_transcription_test(profile)

    def test_name_not_found_suite(self):
        self.plugin_suite_register_err_test(os.path.join(self.source_root, "NameNotFound"))

    def test_missing_file_suite(self):
        self.plugin_suite_register_err_test(os.path.join(self.source_root, "MissingFiles"))

    def plugin_suite_register_err_test(self, suite_source):
        try:
            self.under_test.register_suite(suite_source)
            assert False
        except Exception as e:
            print(e)
            error = self.under_test.report_suite_registration_error(suite_source)
            print(error)

    def plugin_suite_transcription_test(self, profile):
        profile_name = "TestPlugin"
        if self.under_test.is_profile(profile_name):
            self.under_test.remove_profile(profile_name)
        self.under_test.create_profile(
            name=profile_name,
            setting=profile
        )
        small_id = self.under_test.add_source(self.small_source, self.output)
        self.under_test.apply_profile_to_source(source_id=small_id, profile_name=profile_name)
        test2ab_id = self.under_test.add_source(self.test_2ab, self.output)
        self.under_test.apply_profile_to_source(source_id=test2ab_id, profile_name=profile_name)
        result = self.under_test.transcribe([small_id, test2ab_id])
        assert not result.failure
        assert not result.invalid
        assert result.success_output


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2022-08-23 09:38:56
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-05 08:05:03
import setuptools

if __name__ == "__main__":
    setuptools.setup()



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-02-24 10:13:06
# @Email:
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-07 09:47:25
# @Descriptions: convert a conversation directory source to a singleton list of payload

import os.path
from datetime import datetime
from typing import List

from .converterFun import ConvertFun
from .mediaConverter import MediaConverter
from .converter import Converter, ConverterType
from ..payloadObject import PayloadObject
from ..payloadObject.untranscribedDirPayload import untranscribedDirPayload
from ...profileManager import ProfileObject
from ...shared.utils.general import paths_in_dir, get_name
from ...sourceManager.watcher import Watcher
from ...workspace.directory_structure import TemporaryFolder


class ConversationDirConverter(Converter):
    accepted_forms = ["/"]
    converter_type = ConverterType.ConvDir

    @staticmethod
    def is_accepted_form(path: str):
        if not os.path.isdir(path):
            return False
        supported_format = MediaConverter.accepted_forms
        sub_paths = paths_in_dir(path, supported_format, recursive=False)
        if len(sub_paths) == 0:
            return False
        return True

    @staticmethod
    def convert(
        path: str, output_dir: str, profile: ProfileObject, watchers: List[Watcher]
    ) -> List[PayloadObject]:
        name = get_name(path)
        time = datetime.now().strftime("%H-%M-%S-%f")
        temp_path = TemporaryFolder(f"{name}_{time}")
        accepted_files = paths_in_dir(
            path, MediaConverter.accepted_forms, recursive=False
        )
        data_files = []
        for f in accepted_files:
            data_files.extend(ConvertFun.convert_to_wav(f, temp_path.data_copy))

        return [
            untranscribedDirPayload(
                original_source=path,
                data_files=data_files,
                temp_dir_struct=temp_path,
                output_path=output_dir,
                profile=profile,
                watchers=watchers,
            )
        ]



import unittest

from gailbot.shared.exception.serviceException import EngineAddError, DuplicateProfile
from gailbot.shared.exception.transcribeException import InternetConnectionError


class MyTestCase(unittest.TestCase):
    def test_exception(self):
        try:
            raise InternetConnectionError()
        except Exception as e:
            print(e)
            assert isinstance(e, InternetConnectionError)

    def test_add_exception(self):
        try:
            raise EngineAddError("test", cause=Exception())
        except Exception as e:
            print(e)
            assert isinstance(e, EngineAddError)

    def test_duplicate_exception(self):
        try:
            raise DuplicateProfile("test")
        except Exception as e:
            print(e)
            assert isinstance(e, DuplicateProfile)


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from gailbot.pluginSuiteManager.pluginSuiteManager import PluginSuiteManager
from test.suiteManager import SUITE_TEST_WORKSPACE_PATH, SUITE_TEST_SRC_PATH


class TestSuiteManager(unittest.TestCase):
    under_test = PluginSuiteManager(SUITE_TEST_WORKSPACE_PATH)

    def test_report_error(self):
        self.under_test.register_suite(os.path.join(SUITE_TEST_SRC_PATH, "MissingFiles"))
        error = self.under_test.report_registration_err(os.path.join(SUITE_TEST_SRC_PATH, "MissingFiles"))
        print(error)

        self.under_test.register_suite(os.path.join(SUITE_TEST_SRC_PATH, "NameNotFound"))
        error = self.under_test.report_registration_err(os.path.join(SUITE_TEST_SRC_PATH, "NameNotFound"))
        print(error)

        self.under_test.register_suite(os.path.join(SUITE_TEST_SRC_PATH, "UnresolvedDependency"))
        error = self.under_test.report_registration_err(os.path.join(SUITE_TEST_SRC_PATH, "UnresolvedDependency"))
        print(error)




if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# @Author: Vivian Li 
# @Date:   2024-02-23 18:03:11
# @Last Modified by:   Vivian Li 
# @Last Modified time: 2024-02-25 16:19:10
from gailbot import ProfileSetting
from gailbot.shared.exception.serviceException import DuplicateProfile
from gailbot import GailBot


def test_crete_duplicate_profile():
    GailBot().reset_workspace()
    gb = GailBot()
    gb.create_profile(name="test",
                      setting=ProfileSetting(engine_setting_name=gb.default_engine_name(), plugin_suite_setting={}))
    assert gb.is_profile("test")
    assert gb.get_profile_setting("test").engine_setting_name == gb.default_engine_name()

    try:
        gb.create_profile(name="test",
                          setting=ProfileSetting(engine_setting_name=gb.default_engine_name(), plugin_suite_setting={}))
        assert False
    except Exception as e:
        assert isinstance(e, DuplicateProfile)


def test_update_profile():
    GailBot().reset_workspace()
    gb = GailBot()
    gb.create_profile(name="test",
                      setting=ProfileSetting(engine_setting_name=gb.default_engine_name(), plugin_suite_setting={}))
    assert gb.is_profile("test")
    assert gb.get_profile_setting("test").engine_setting_name == gb.default_engine_name()
    assert not gb.get_profile_setting("test").plugin_suite_setting.keys()

    gb.update_profile(profile_name="test",
                      new_setting=gb.get_profile_setting(gb.profile_manager.default_profile_name))
    assert gb.get_profile_setting("test").plugin_suite_setting.keys()





# -*- coding: utf-8 -*-
# @Author: Jason Y. Wu
# @Date:   2023-07-17 19:21:51
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-09 18:34:34
from dataclasses import dataclass


@dataclass
class PLUGIN_CONFIG:
    EN_KEY = b"25T_bNAjTALUgva1C-au0IKwESptsLAztLRVVEyCtZs="
    ENCRYPTED_API_ID = b"gAAAAABkU5CF3GwKfqwPBvgG4fibAHjZ2hE1pQdEb8Nkw06YubGZ9_mekBRgMiRXPrXo7C_6i-uuBpisZyL3FpFwg8zXvxR9_jrPocUkMjSmyHFJQUYojys="
    ENCRYPTED_API_KEY = b"gAAAAABkU5CF1Q_ebrx9OTuW0WWQypQKTk25JPPIrRGAyoDLgweyh-2tDnD4CdmUE_xpprxVMttgvz7FzR2sDMn6jJ3EyYNxUCHgudOBkwPoYYM7a6fMheNLlyT3x_SzszpnxfF2y7-m"
    THREAD_NUM = 3
    REQUIREMENT = "requirements.txt"
    OFFICIAL = ".gb_official"
    CONFIG = "suite_info.toml"
    FORMAT = "format.md"
    OFFICIAL_KEY = b"KM1V0_HijdrehPVsFueXISED2Jxs0S_rR9zKAJzcW3A="
    OFFICIAL_ENKEY = b"q6LGp_5jjEQyZu3ydl88H2adl_RRZCyw4IMXqYXnga8="



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-02-24 10:13:16
# @Email:
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-02 18:01:08
# @Descriptions: convert a directory source to a list of corresponding payloads


import os.path
from typing import List

from gailbot.payload.payloadConverter.mediaConverter import MediaConverter
from gailbot.payload.payloadConverter.conversationDirConverter import (
    ConversationDirConverter,
)
from gailbot.payload.payloadConverter.converter import Converter, ConverterType
from gailbot.payload.payloadConverter.transcribedDirConverter import (
    TranscribedDirConverter,
)
from gailbot.payload.payloadObject import PayloadObject
from gailbot.profileManager import ProfileObject
from gailbot.shared.utils.general import paths_in_dir
from gailbot.sourceManager.watcher import Watcher


class MixedDirectoryConverter(Converter):
    accepted_forms = ["/"]
    converter_type = ConverterType.MixedDirectory

    @staticmethod
    def convert(
        path: str, output_dir: str, profile: ProfileObject, watchers: List[Watcher]
    ) -> List[PayloadObject]:
        converters: List = [
            TranscribedDirConverter,
            ConversationDirConverter,
            MediaConverter,
            MixedDirectoryConverter,
        ]
        payloads = []
        subpaths = paths_in_dir(path)
        for p in subpaths:
            for converter in converters:
                if converter.is_accepted_form(p):
                    payloads.extend(converter.convert(p, output_dir, profile, watchers))
                    break
        return payloads

    @staticmethod
    def is_accepted_form(path: str) -> bool:
        supported_format = MediaConverter.accepted_forms
        return (
            os.path.isdir(path)
            and paths_in_dir(path, supported_format, True) != 0
            and not TranscribedDirConverter.is_accepted_form(path)
        )



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-02-25 08:58:26
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-03-26 15:21:13
import os.path
from dataclasses import dataclass


@dataclass
class TestData:
    OUTPUT = "/Users/yike/Desktop/gbtest/output"
    INPUT = "/Users/yike/Desktop/gbtest/input"
    BAD_INPUT = os.path.join(INPUT, "media_formats")
    MIX_DIR = os.path.join(INPUT, "mixed_dir")
    hello = os.path.join(INPUT, "hello2.wav")
    test2ab = os.path.join(INPUT, "test2ab")
    test2ab_transcribed = os.path.join(INPUT, "test2ab_transcribed")
    test0638_transcribed = os.path.join(INPUT, "0638-transcribed")
    small_dir2 = os.path.join(INPUT, "small_test2")
    small_dir4 = os.path.join(INPUT, "small_test4")
    small_dir6 = os.path.join(INPUT, "small_test6")
    test_mp3 = os.path.join(INPUT, "test.mp3")
    callhome_small = os.path.join(INPUT, "callhome_small")
    callhome_full = os.path.join(INPUT, "callhome_full")



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-31 11:09:26
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-02 17:48:48


import os
from threading import Lock
import json
from typing import List, Dict
from dataclasses import asdict

import torch
import whisper_timestamped as whisper
from whisper_timestamped.transcribe import force_cudnn_initialization

from gailbot.workspace import WorkspaceManager
from .parsers import (
    parse_into_full_text,
)

from .parsers import parse_into_word_dicts
from gailbot.shared.utils.general import is_file, make_dir
from gailbot.configs import whisper_config_loader
from gailbot.shared.utils.logger import makelogger

logger = makelogger("whisper")

WHISPER_CONFIG = whisper_config_loader()
LOAD_MODEL_LOCK = Lock()


class WhisperCore:
    """
    We are using this class as an adapter for the engine so that we can use
    multiple different instances of the underlying whisper package is required.
    """

    # NOTE: Intentionally limiting the supported formats since we have
    # not tested other formats.
    _SUPPORTED_FORMATS = "wav"

    def __init__(self):
        # initialize the workspace
        self.workspace_dir = os.path.join(WorkspaceManager.engine_src, "whisper")
        self.cache_dir = os.path.join(self.workspace_dir, "cache")
        self.models_dir = os.path.join(self.cache_dir, "models")
        self.loadModelLock = Lock()
        make_dir(self.workspace_dir, overwrite=False)
        make_dir(self.cache_dir, overwrite=False)
        make_dir(self.models_dir, overwrite=False)

        # Load a GPU is it is available
        self.device = "cuda:0" if torch.cuda.is_available() else "cpu"
        if self.device.lower().startswith("cuda"):
            force_cudnn_initialization(self.device)
        logger.info(f"Whisper core initialized with device: {self.device}")

    def __repr__(self) -> str:
        configs = json.dumps(
            asdict(WHISPER_CONFIG.transcribe_configs), indent=2, ensure_ascii=False
        )
        return (
            f"Whisper model: {WHISPER_CONFIG.model_name}"
            f"Transcribe configs:\n{configs}"
        )

    def transcribe(
        self, audio_path: str, language: str = None, detect_speaker: bool = False
    ) -> List[Dict]:
        """
        Use the engine to transcribe an item
        """
        assert is_file(audio_path), f"ERROR: Invalid file path: {audio_path}"
        global LOAD_MODEL_LOCK
        with LOAD_MODEL_LOCK:
            whisper_model = whisper.load_model(
                name=WHISPER_CONFIG.model_name,
                device=self.device,
                download_root=self.models_dir,
            )
        logger.info(f"Whisper core using whisper model: {WHISPER_CONFIG.model_name}")

        if language is None:
            logger.info("no language specified - auto detecting language")

        # Load the audio and models, transcribe, and return the parsed result
        audio = whisper.load_audio(audio_path)

        asr_result = whisper.transcribe(
            whisper_model,
            audio,
            language=language,
            **asdict(WHISPER_CONFIG.transcribe_configs),
        )

        if WHISPER_CONFIG.transcribe_configs.verbose:
            logger.debug(parse_into_full_text(asr_result))

        res = parse_into_word_dicts(asr_result)
        return res

    def get_supported_formats(self) -> List[str]:
        return list(self._SUPPORTED_FORMATS)

    def get_available_models(self) -> List[str]:
        return whisper.available_models()



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-08 13:22:01
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-21 14:21:16
# @Description: PluginManger stores available plugin suites in a map from suite
# name to PluginSuite object, and provides method to load, delete, and reload
# plugin suite
import platform
import shutil

import sys
import os
from datetime import datetime
from typing import List, Dict
from gailbot.pluginSuiteManager.S3BucketManager import S3BucketManager

from gailbot.shared.exception.serviceException import (
    FailReportError,
    PluginSuiteNotFound,
    RemoveOfficialPlugin,
)
from gailbot.shared.utils.download import is_internet_connected
from gailbot.shared.utils.logger import makelogger
from gailbot.shared.utils.general import (
    get_subfiles,
    make_dir,
    subdirs_in_dir,
    delete,
    get_name,
    is_directory,
    num_subdirs,
)
from gailbot.configs.interfaces import get_plugin_structure_config
from gailbot.configs import PLUGIN_CONFIG
from gailbot.pluginSuiteManager.suite.suite import PluginSuite
from gailbot.pluginSuiteManager.suiteLoader import (
    PluginDirectorySuiteLoader,
    PluginURLSuiteLoader,
    PluginSuiteLoader,
)
from gailbot.pluginSuiteManager.suite.pluginData import Suite
from gailbot.pluginSuiteManager.APIConsumer import APIConsumer

EXPECTED_STRUCTURE = get_plugin_structure_config()
logger = makelogger("plugin_manager")


class PluginSuiteManager:
    """
    Manage multiple plugins suites that can be registered, including
    storing the plugin files, parsing the config files, and instantiating
    plugin objects from files.
    """

    def __init__(self, workspace: str):
        """
        Args:
        workspace (str) : the path to plugin workspace
        loading_existing: if true, load the existing plugin from plugin workspace
        """

        self.workspace = workspace
        self._init_workspace()
        self.suites = dict()
        self.logs = dict()  # map source string to registration log
        self.loaders: List[PluginSuiteLoader] = [
            PluginURLSuiteLoader(self.download_dir, self.suites_dir),
            PluginDirectorySuiteLoader(self.suites_dir),
        ]

        # download all the suites in HiLab bucket upon first launch of gailbot
        try:
            self._load_existing_suite_from_workspace()
        except Exception as e:
            logger.error(e, exc_info=True)
        self.paths_to_existing_suites = subdirs_in_dir(self.suites_dir, recursive=True)

    def _load_existing_suite_from_workspace(self):
        #TODO: this function currently downloads all the suites in self.suites_dir, 
        # but we need it to only download the default HiLabSuite and gb_hilab_suite
        # or we can change it to take in the name of the suite to download
        
        subdirs = subdirs_in_dir(self.suites_dir, recursive=False)
        directory_loader = PluginDirectorySuiteLoader(self.suites_dir)

        for plugin_source in subdirs:
            register_file = os.path.join(
                self.log_dir, "log" + datetime.now().strftime("%m-%d-%H-%M-%f")
            )
            self.logs[plugin_source] = register_file
            f = open(register_file, "a")
            suites = directory_loader.load(plugin_source, f=f)
            if not suites:
                logger.error(f"{get_name(plugin_source)} cannot be registered")
            else:
                for suite in suites:
                    if isinstance(suite, PluginSuite):
                        self.suites[suite.name] = suite
            f.close()

    # Returns true if aws has a newer version of the given suite. Otherwise,
    # false.
    def has_new_version(self, suite_name) -> bool:
        if not is_internet_connected():
            return False
        try:
            bucket_metadata = S3BucketManager.get_instance()
            local_metadata = self.get_suite_metadata(suite_name)
            remote_version = bucket_metadata.get_remote_version(
                local_metadata.BucketName, local_metadata.ObjectName
            )

            return remote_version != local_metadata.Version
        except Exception as e:
            logger.error(e, exc_info=e)
            return False

    # Returns the names of the files in the suite directory that were modified
    # after initial creation time.
    def modified_files_in_suite(self, suite_name) -> List[str]:
        ignore_files = ["__pycache__", ".DS_Store", "config.toml"]
        try:
            suite_path = os.path.join(self.suites_dir, suite_name)
            suite_file = os.stat(suite_path)

            if platform.system() == "Windows":
                creation_time = suite_file.st_mtime
            else:
                creation_time = suite_file.st_birthtime
            suite_creation_time_datetime = datetime.fromtimestamp(
                creation_time
            ).replace(second=0, microsecond=0)
            modified_files = []
            # Store the names of the files that have been changed after creation.
            for file_path in get_subfiles(path=suite_path):
                if any(ignore in file_path for ignore in ignore_files):
                    continue  # Skip this file if it matches any ignore pattern
                file_timestamp = os.path.getmtime(file_path)
                file_modification_datetime = datetime.fromtimestamp(
                    file_timestamp
                ).replace(second=0, microsecond=0)
                if suite_creation_time_datetime != file_modification_datetime:
                    modified_files.append(file_path)

            return modified_files
        except Exception as e:
            logger.error(e, exc_info=e)
            return []

    # Makes a copy of the given suite in the suite_copies folder.
    def copy_modified_suite(self, suite_name):
        source_dir = os.path.join(self.suites_dir, suite_name)
        base_destination_dir_name = "modified-" + suite_name
        destination_dir = os.path.join(self.copy_suite_dir, base_destination_dir_name)

        counter = 1
        while os.path.exists(destination_dir):
            versioned_destination_dir_name = f"{base_destination_dir_name}-{counter}"
            destination_dir = os.path.join(
                self.copy_suite_dir, versioned_destination_dir_name
            )
            counter += 1
        try:
            shutil.copytree(source_dir, destination_dir)
        except Exception as e:
            logger.error(e, exc_info=e)

    def reload_suite(self, suite_name):
        """
        re-load the suite from the original source

        Parameters
        ----------
        suite_name: str
            the name of the suite

        Returns
        ----------

        Raises
        ------
        FailPluginSuiteReload
        """
        try:
            object_url = self.get_suite_metadata(suite_name).ObjectUrl
            self.register_suite(object_url)
            self.paths_to_existing_suites = subdirs_in_dir(
                self.suites_dir, recursive=True
            )
        except Exception as e:
            logger.error(e, exc_info=e)

    def get_all_suites_name(self) -> List[str]:
        return list(self.suites.keys())

    def is_suite(self, suite_name: str) -> bool:
        return suite_name in self.suites

    def register_suite(self, plugin_source: str) -> List[str]:
        """
        Register a plugin suite from the given source, which can be
        a plugin directory, an url, a conf file, or a dictionary configuration.
        """
        register_file = os.path.join(
            self.log_dir, "log" + datetime.now().strftime("%m-%d-%H-%M-%f")
        )
        self.logs[plugin_source] = register_file
        f = open(register_file, "a")

        registered = []
        try:
            # self.loader holds PluginURLLoader and PluginDirectoryLoader
            for loader in self.loaders:
                suites = loader.load(plugin_source, f)
                # PluginURLLoader's load function isn't implemented
                # PluginDirectoryLoader returns a list with one item
                # when load is called with PluginDirectory loader, the following happens:
                # 1) gets and loads requirements
                # 2) make a copy of the original plugin suite and saves
                # it as suite, the function returns [suite]
                # 3) validates the suite
                # if the anything fails (ex: the directory doesn't exist)
                # boolean false value is returned
                if suites:
                    for suite in suites:
                        if isinstance(suite, PluginSuite):
                            self.suites[suite.name] = suite
                            registered.append(suite.name)
            f.close()
            return registered
        except Exception as e:
            f.write(f"{e}\n")
            f.close()
            logger.error(e)
            return []

    def get_selectable_plugins(self, suite_name: str) -> List[str]:
        """
        Given a suite_name, returns a list of selectable plugins from that
        suite
        Parameters
        ----------
        suite_name

        Returns
        -------

        Raises
        ------
        PluginSuiteNotFound
        """
        return self.get_suite(suite_name).optional_plugins

    def get_suite(self, suite_name: str) -> PluginSuite:
        """Given a suite name, return the plugin suite object

        Parameters
        ----------
            suite_name (str): the name that identifies the plugin suite

        Returns
        -------
            PluginSuite: the plugin suite object identified by suite name

        Raises
        -------
        PluginSuiteNotFound
        """
        if not self.is_suite(suite_name):
            raise PluginSuiteNotFound(suite_name)
        return self.suites[suite_name]

    def is_official_suite(self, suite_name) -> bool:
        """
        given a suite name, checks if it is an official suite

        Parameters
        ----------
        suite_name: str: the name of the suite

        Returns
        -------
        bool: True if the suite is official

        Raises
        -------
        PluginSuiteNotFound
        """
        if not self.is_suite(suite_name):
            raise PluginSuiteNotFound(suite_name)
        return self.suites[suite_name].is_official

    def get_suite_metadata(self, suite_name: str) -> Suite:
        """
        given a suite name, returns its metadata

        Parameters
        ----------
        suite_name: str: the name of the suite

        Returns
        -------
        MetaData: the metadata or None if unsuccessful

        Raises
        -------
        PluginSuiteNotFound
        """
        if not self.is_suite(suite_name):
            raise PluginSuiteNotFound(suite_name)
        return self.suites[suite_name].get_meta_data()

    def get_suite_dependency_graph(self, suite_name: str) -> Dict[str, List[str]]:
        """
        given a suite name, returns its dependency graph

        Parameters
        ----------
        suite_name: str: the name of the suite

        Returns
        -------
        Dict[str, List[str]]: the dependency graph

        Raises
        -------
        PluginSuiteNotFound
        """
        if not self.is_suite(suite_name):
            raise PluginSuiteNotFound(suite_name)
        return self.suites[suite_name].dependency_graph()

    def get_suite_documentation_path(self, suite_name) -> str:
        """
        given a suite name, returns its documentation path

        Parameters
        ----------
        suite_name: str: the name of the suite

        Returns
        -------
        str: the path to the documentation

        Raises
        -------
        PluginSuiteNotFound
        """
        if not self.is_suite(suite_name):
            raise PluginSuiteNotFound(suite_name)
        return self.suites[suite_name].document_path

    def _init_workspace(self):
        """
        Init workspace and load plugins from the specified sources.
        """
        self.suites_dir = os.path.join(self.workspace, "suites")
        self.copy_suite_dir = os.path.join(self.workspace, "suite_copies")
        self.download_dir = os.path.join(self.workspace, "download")
        self.log_dir = os.path.join(self.workspace, "register_log")
        self.suites: Dict[str, PluginSuite] = dict()

        # appending the path of suite directory root in order for python to import
        # plugin packages
        sys.path.append(self.suites_dir)

        # Make the directory
        make_dir(self.workspace, overwrite=False)
        make_dir(self.suites_dir, overwrite=False)
        make_dir(self.download_dir, overwrite=True)
        make_dir(self.log_dir, overwrite=True)

    def delete_suite(self, name: str):
        """
        given a suite name, delete the plugin suite

        Parameters
        ----------
        name: str: the name of the suite to delete

        Returns
        -------
        bool: true if successfully removed, false otherwise

        Raises
        -------
        PluginSuiteNotFound
        """
        if not self.is_suite(name):
            raise PluginSuiteNotFound(name)
        if self.is_official_suite(name):
            raise RemoveOfficialPlugin()

        delete(os.path.join(self.suites_dir, name))
        del self.suites[name]

    def delete_suites(self, suite_names: List[str]):
        """
        given a list of suite names, delete each plugin suite
        """
        for suite_name in suite_names:
            if not self.delete_suite(suite_name):
                return False
        return True

    def get_suite_path(self, name: str) -> str:
        """
        Given a name of the suite, return the suite path that is internally
        managed by the suite manager

        Parameters
        ----------
        name: str: the name of the suite path

        Returns
        -------
        str: the path of the suite

        Raises
        -------
        PluginSuiteNotFound
        """
        if self.is_suite(name):
            path = os.path.join(self.suites_dir, name)
            if is_directory(path):
                return path
            else:
                raise PluginSuiteNotFound(name)
        else:
            raise PluginSuiteNotFound(name)

    def report_registration_err(self, suite_source: str) -> str:
        """report plugin registration error

        Parameter
        --------
            suite_source (str): the path to the suite source

        Return
        -------
            str: a string that report the plugin registration error

        Raises
        -------
        FailReportError
        """
        try:
            log = self.logs[suite_source]
            with open(log, "r") as f:
                s = f.read()
            return s
        except Exception as e:
            raise FailReportError(suite_source, e)

    def get_suite_url_from_id(self, id):
        api = APIConsumer.get_instance()
        suite_info = api.fetch_suite_info(suite_id=id)
        suite_url = suite_info.get("s3_url", "")

        return suite_url


import os
from pathlib import Path  
from deepgram import DeepgramClient, PrerecordedOptions    
from gailbot.engineManager.engine.engine import Engine
from .deepgramSetting import DeepgramSetting
from typing import Dict, List
from gailbot.shared.utils.logger import makelogger
import requests

logger = makelogger("deepgram engine")

class Deepgram(Engine):
    def __init__(self, setting: DeepgramSetting):
        self.setting = setting
        self.dg = DeepgramClient(self.setting.deepgram_api_key)
        self.transcribe_success = False

    def __repr__(self):
        return self.setting.engine

    def transcribe(self, audio_path: str, workspace: str):
        dg = DeepgramClient(self.setting.deepgram_api_key)

        with open(audio_path, "rb") as audio:
            source = {"buffer": audio, "mimetype": "audio/wav"} 
            options = PrerecordedOptions(
                model=self.setting.model,
                punctuate=self.setting.punctuate,
                diarize=self.setting.diarize,
                profanity_filter=self.setting.profanity_filter,
                language=self.setting.language,
            )

            response = dg.listen.rest.v("1").transcribe_file(source, options)  # sync


            ws_path = Path(workspace)
            ws_path.mkdir(parents=True, exist_ok=True)       
            out_json = ws_path / f"{Path(audio_path).stem}.deepgram.json"
            out_json.write_text(response.to_json(indent=2))

            utterances = [
                {
                    "start": w.start,
                    "end":   w.end,
                    "text":  w.word,
                    "speaker": w.speaker or "",
                }
                for ch in response.results.channels
                for alt in ch.alternatives
                for w   in alt.words
            ]

            self.transcribe_success = True
            return utterances

    def is_file_supported(self, file_path: str) -> bool:
        ext = os.path.splitext(file_path)[1].lower()
        return ext in ['.wav', '.mp3', '.flac', '.m4a', '.ogg']

    def get_supported_formats(self) -> List[str]:
        return ['wav', 'mp3', 'flac', 'm4a', 'ogg']

    def get_engine_name(self) -> str:
        return self.setting.engine

    def was_transcription_successful(self) -> bool:
        return self.transcribe_success

    @staticmethod
    def is_valid_deepgram_api(api_key: str, timeout: float = 5.0) -> bool:
        """
        Validate a Deepgram API key by calling the /v1/auth/token endpoint.
        Returns True if the key is valid (HTTP 200), False otherwise.
        """
        url = "https://api.deepgram.com/v1/auth/token"
        headers = {"Authorization": f"Token {api_key}"}
        try:
            resp = requests.get(url, headers=headers, timeout=timeout)
            return resp.status_code == 200
        except requests.RequestException:
            # Network error or timeout
            return False



import gailbot
from gailbot.engineManager.engine.whisperX.whisperX import WhisperX
from gailbot.engineManager.engine.whisperX.whisperXSetting import WhisperXSetting

setting = WhisperXSetting()
# setting.engine = "whisperX"
# setting.output_format = "all"
# setting.language = "en"
# setting.interpolate_method = "nearest"
# setting.min_speakers = 1
# setting.max_speakers = 2
# setting.temperature = 1
# setting.condition_on_previous_text = False
# setting.initial_prompt = None
# setting.vad_onset = 0.5
# setting.vad_offset = 0.363
# setting.chunk_size =  30
# setting.no_speech_threshold = 0.6
print("created")

whisp = WhisperX(setting)

audio = "/Users/evacaro/Desktop/test.wav"
payload = "/Users/evacaro/GailBot/gailbot_workspace/temporary/test_0618_11_14_09_068129/transcribe_ws"

transcription = whisp.transcribe(audio_path= audio, payload_workspace= payload)

print("done ", transcription)


# -*- coding: utf-8 -*-
# @Author: Your name
# @Date:   2024-03-01 16:47:44
# @Last Modified by:   Your name
# @Last Modified time: 2024-03-06 18:53:15
from .payloadObject.transcribedDirPayload import TranscribedDirPayload
from .payloadObject.untranscribedDirPayload import untranscribedDirPayload
from .payloadObject.payloadObject import PayloadObject
from .payloadConverter.converterFun import ConvertFun



from gailbot.transcriptionPipeline.pipeline import GBPipeline
from gailbot.sourceManager.sourceObject import SourceObject
from gailbot.profileManager.profileManager  import ProfileObject
from gailbot.engineManager.engine.engineProvider import EngineProvider
from gailbot.engineManager.engine.whisperX.whisperXProvider import WhisperXProvider
from gailbot.engineManager.engine.whisperX.whisperXSetting import WhisperXSetting


gb = GBPipeline()
source = "/Users/evacaro/Music/Music/Media.localized/Unknown Artist/Unknown Album/test.wav"
output = "/Users/evacaro/Desktop/test_out_2"
source_id = "test_analyze_name"

src_object = SourceObject(source_id= source_id, source_path= source, output= output)
setting = WhisperXSetting()
eng_prov = WhisperXProvider(name= "whisperxprov",  data= setting)
prof = ProfileObject(name= "test_eng_prov", profile_data= None, engine_provider= eng_prov, plugin_suites= None)
prof.name = "test_prof_name"
prof.profile_data = None
prof.plugin_suites = None
prof.engine_provider = eng_prov

src_object.initialize_profile(prof)
src_list = [SourceObject(source_id= source_id, source_path= source, output= output)]
print("about to convert\n")
convert_res = gb.convert(sources= src_list)
print("printing og source ids: ")
for id in convert_res.original_source_ids:
    print(id)



# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-03-30 11:57:13
# @Last Modified by:   Vivian Li 
# @Last Modified time: 2024-05-03 17:16:44
import os.path
from typing import Dict, Optional

from gailbot.engineManager.engine.engine import Engine
from gailbot.engineManager.engine.engineProvider import EngineProvider
from gailbot.engineManager.engine.google import Google
from gailbot.engineManager.engine.google.googleSetting import GoogleSetting
from gailbot.shared.utils.general import make_dir, copy
from gailbot.shared.utils.logger import makelogger
from gailbot.workspace import WorkspaceManager

logger = makelogger("google setting")


class GoogleProvider(EngineProvider):
    workspace = os.path.join(WorkspaceManager.engine_src, "google/api")

    def is_cpu_intensive(self) -> bool:
        return False

    @staticmethod
    def load(name, data: Dict[str, str]) -> Optional[EngineProvider]:
        try:
            engine = GoogleProvider(name, data)
            assert engine.data.engine.lower() == "google"
            return engine
        except Exception as e:
            logger.warning(e)
            return None

    def __init__(self, name, data: Dict[str, str]):
        if not os.path.isdir(self.workspace):
            make_dir(self.workspace)
        self.name = name
        self.data: GoogleSetting = GoogleSetting(**data)
        assert Google.is_valid_google_api(self.data.google_api_key)
        copied_path = os.path.join(
            self.workspace, os.path.basename(self.data.google_api_key)
        )
        if not os.path.isfile(copied_path):
            copy(src_path=self.data.google_api_key, tgt_path=copied_path)

        self.data.google_api_key = copied_path

    def make_engine(self) -> Engine:
        return Google(self.data)



import os.path
import unittest

from gailbot.workspace import WorkspaceManager


class TestWorkspaceManager(unittest.TestCase):
    def test_init_workspace(self):
        ws = WorkspaceManager()
        ws.init_workspace()
        assert os.path.getctime(WorkspaceManager.gb_root) >= WorkspaceManager._last_update
        for p in WorkspaceManager.paths:
            assert os.path.isdir(p)

    def test_reset_workspace(self):
        ws = WorkspaceManager()
        ws.init_workspace()
        for p in WorkspaceManager.paths:
            assert os.path.isdir(p)
        orig_ws_creation_time = os.path.getctime(WorkspaceManager.gb_root)
        ws.reset_workspace()
        for p in WorkspaceManager.paths:
            assert os.path.isdir(p)
        reset_ws_creation_time = os.path.getctime(WorkspaceManager.gb_root)
        assert orig_ws_creation_time != reset_ws_creation_time


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-06-15 15:27:03
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-09 18:39:41
import os.path
from dataclasses import dataclass

import userpaths

from gailbot.shared.utils.general import is_directory, make_dir, delete
from gailbot.shared.utils.logger import makelogger

logger = makelogger("workspace")


@dataclass
class WorkspaceManager:
    """store the path data of the workspace, provide utility function to
    create temporary and output directories
    """

    _last_update = 1708863997.0  # any GailBot folder older than this timestamp will be reinitialized
    gb_root = os.path.join(userpaths.get_profile(), "GailBot")
    gb_ws = os.path.join(gb_root, "gailbot_workspace")
    temporary_ws = os.path.join(gb_ws, "temporary")
    gb_data = os.path.join(gb_ws, "gailbot_data")
    plugin_src = os.path.join(gb_data, "plugin_source")
    suites = os.path.join(plugin_src, "suites")
    plugins = os.path.join(plugin_src, "plugins")
    engine_src = os.path.join(gb_data, "engine_source")
    engine_setting_src = os.path.join(engine_src, "engine_setting")
    profile_src = os.path.join(gb_data, "profile_source")
    paths = [gb_root, gb_ws, temporary_ws, gb_data, plugin_src, engine_src, profile_src]

    def init_workspace(self):
        """
        Initializes the workspace
        """
        if (
            is_directory(self.gb_root)
            and os.path.getctime(self.gb_root) < self._last_update
        ):
            delete(self.gb_root)
        for path in self.paths:
            if not is_directory(path):
                make_dir(path, True)

    def reset_workspace(self):
        """
        Resets the given workspace

        Returns
        -------
        True if successful, false otherwise
        """
        try:
            if is_directory(self.gb_root):
                delete(self.gb_root)
            self.init_workspace()
            return True
        except Exception as e:
            logger.error(e, exc_info=e)
            return False

    def clear_gb_temp_dir(self):
        """
        Clears the temporary workspace directory

        Returns
        -------
        False if exception is raised
        """
        try:
            if is_directory(self.temporary_ws):
                delete(self.temporary_ws)
        except Exception as e:
            logger.error(e, exc_info=e)
            return False



# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-08 12:25:27
# @Last Modified by:   Muhammad Umair
# @Last Modified time: 2023-01-10 14:21:38

from .component import Component, ComponentState, ComponentResult
from .pipeline import Pipeline